from webull import webull as wb
from typing import List, Any
import subprocess
from plotly.subplots import make_subplots
from apis.polygonio.polygon_database import PolygonDatabase
poly = PolygonDatabase(user='chuck', database='markets')
from kaleido.scopes.plotly import PlotlyScope
from concurrent.futures import ThreadPoolExecutor
import asyncio
PLT_3DMESH_HOVERLABEL = dict(bgcolor="gold")
PLT_3DMESH_STYLE_TEMPLATE = "plotly_dark"

scope = PlotlyScope(
    plotlyjs="plotly.js"),
# Chart Plots Settings
PLT_CANDLE_STYLE_TEMPLATE = "plotly_dark"
PLT_CANDLE_INCREASING = "#00ACFF"
PLT_CANDLE_DECREASING = "#e4003a"
PLT_CANDLE_VOLUME = "#fdc708"
PLT_CANDLE_NEWS_MARKER = "rgba(255, 215, 0, 0.9)"
PLT_CANDLE_NEWS_MARKER_LINE = "gold"
PLT_CANDLE_YAXIS_TEXT_COLOR = "#fdc708"
PLT_SCAT_STYLE_TEMPLATE = "plotly_dark"
PLT_SCAT_INCREASING = "#00ACFF"
PLT_SCAT_DECREASING = "#e4003a"
PLT_SCAT_PRICE = "#fdc708"
PLT_TA_STYLE_TEMPLATE = "plotly_dark"
PLT_FONT = dict(
    family="Fira Code",
    size=12,
)
PLT_TA_COLORWAY = [
    "#fdc708",
    "#d81aea",
    "#00e6c3",
    "#9467bd",
    "#e250c3",
    "#d1fa3d",
]
PLT_FIB_COLORWAY: List[Any] = [
    "rgba(195, 50, 69, 1)",  # 0
    "rgba(130, 38, 96, 1)",  # 0.235
    "rgba(72, 39, 124, 1)",  # 0.382
    "rgba(0, 93, 168, 1)",  # 0.5
    "rgba(173, 0, 95, 1)",  # 0.618
    "rgba(253, 199, 8, 1)",  # 0.65 Golden Pocket
    "rgba(147, 103, 188, 1)",  # 1
    dict(
        family="Fire Code",
        size=16,
    ),  # Fib's Text
    dict(color="rgba(0, 230, 195, 1)", width=2, dash="dash"),  # Fib Trendline
]

import plotly.graph_objects as go
from selenium import webdriver
from selenium.webdriver.chrome.options import Options
import os
import matplotlib.pyplot as plt
import seaborn as sns
from mpl_toolkits.mplot3d import Axes3D
import numpy as np

import pandas as pd
import aiohttp
from aiohttp.client_exceptions import ClientConnectorError

# Import necessary libraries
import plotly.express as px
# Function to abbreviate numbers
def abbreviate_number(num):
    if num >= 1000000:
        return f"{num/1000000:.1f}M"
    elif num >= 1000:
        return f"{num/1000:.1f}K"
    else:
        return str(num)

async def plot_td9_chart(df, interval):
    df.index = df['Timestamp']
    df.index = pd.to_datetime(df['Timestamp'])
    fig = go.Figure()
    fig = make_subplots(rows=2, cols=1, shared_xaxes=True, row_heights=[0.8, 0.2], vertical_spacing=0.02)


    # Add Candlestick trace
    fig.add_trace(
        go.Candlestick(
            x=df.index.strftime('%Y-%m-%d %H:%M:%S'),  # Format timestamp
            open=df['Open'],
            high=df['High'],
            low=df['Low'],
            close=df['Close'],
            increasing_line_color=PLT_CANDLE_INCREASING,
            decreasing_line_color=PLT_CANDLE_DECREASING,
            name='OHLC'
        )
    )
    ticker = df['ticker']

    # Add Volume as a bar chart
    fig.add_trace(
        go.Bar(
            x=df.index.strftime('%Y-%m-%d %H:%M:%S'),  # Format timestamp
            y=df['Volume'],
            name='Volume',
            marker_color=PLT_CANDLE_VOLUME,
            yaxis='y2'
        )
    )



    # Apply your plot styles
    fig.update_layout(
        template=PLT_CANDLE_STYLE_TEMPLATE,
        title=f'TD9 Chart with RSI - {ticker} | {interval}',
        yaxis=dict(domain=[0.3, 1]),
        yaxis2=dict(domain=[0, 0.2]),
    )

    

    fig.write_html('temp_plot.html')
    # Get the absolute path of the HTML file
    file_path = os.path.abspath('temp_plot.html')
    # Selenium to capture image
    options = webdriver.ChromeOptions()


    options.add_argument('--headless')  # This argument enables headless mode
    options.add_argument('--ignore-certificate-errors')
    options.add_argument('--allow-running-insecure-content')
    options.add_argument('--proxy-bypass-list=<-loopback>;www.sec.gov')
    options.add_argument('--ignore-ssl-errors=yes')
    options.add_argument('--ignore-certificate-errors-spki-list')
    options.add_argument("user-agent=Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/115.0.0.0 Safari/537.36")
    # Initialize WebDriver
    browser = webdriver.Chrome(options=options)


    browser.get(f'file://{file_path}')

    # Take screenshot
    screenshot_bytes = browser.get_screenshot_as_png()

    # Save the screenshot to a file (optional)
    with open('screenshot.jpg', 'wb') as f:
        f.write(screenshot_bytes)

    # Close the browser
    browser.quit()
    return screenshot_bytes


async def plot_oi_volume(all_options_df):
    # Apply the abbreviation function to the 'vol' and 'OI' columns
    agg_data = all_options_df.groupby('strike').agg({'vol': 'sum', 'OI': 'sum'}).reset_index()
    agg_data['vol_abbrev'] = agg_data['vol'].apply(abbreviate_number)
    agg_data['OI_abbrev'] = agg_data['OI'].apply(abbreviate_number)

    # Create the bar chart using Plotly
    fig = go.Figure()

    # Add Volume bars
    fig.add_trace(go.Bar(
        x=agg_data['strike'],
        y=agg_data['vol'],
        name='Volume',
        text=agg_data['vol_abbrev'],
        textposition='inside',
        marker=dict(color='blue')
    ))

    # Add OI bars
    fig.add_trace(go.Bar(
        x=agg_data['strike'],
        y=agg_data['OI'],
        name='Open Interest',
        text=agg_data['OI_abbrev'],
        textposition='inside',
        marker=dict(color='green')
    ))

    # Update the layout
    fig.update_layout(
        title='Consolidated OI and Volume by Strike',
        titlefont=dict(size=20),
        xaxis_title='Strike',
        yaxis_title='Aggregated Value',
        barmode='group',
        bargap=0.9,  # gap between bars of adjacent location coordinates
        bargroupgap=0.1,  # gap between bars of the same location coordinate
        template='plotly_dark'
    )



async def plot_calls_and_puts(ticker, underlying_price):
    call_all_options_df = await poly.get_option_chain_all(ticker, contract_type='call')
    put_all_options_df = await poly.get_option_chain_all(ticker, contract_type='put')


    call_data = call_all_options_df.df
    put_data = put_all_options_df.df

    # Calculate the underlying price (assuming it's a constant for all rows, hence taking the first row's value)
  

    # Calculate the range for 25% from the money
    lower_bound = underlying_price * 0.75
    upper_bound = underlying_price * 1.25

    # Filter the call and put data to include only the strikes within the specified range
    call_data_filtered = call_data[(call_data['strike'] >= lower_bound) & (call_data['strike'] <= upper_bound)]
    put_data_filtered = put_data[(put_data['strike'] >= lower_bound) & (put_data['strike'] <= upper_bound)]

    # Create the zoomed-in bar chart
    fig = go.Figure()

    # Add Call Volume bars
    fig.add_trace(go.Bar(
        x=call_data_filtered['strike'],
        y=call_data_filtered['vol'],
        name='Call Volume',
        text=call_data_filtered['vol_abbrev'],
        textposition='inside',
        marker=dict(color='blue')
    ))

    # Add Put Volume bars
    fig.add_trace(go.Bar(
        x=put_data_filtered['strike'],
        y=put_data_filtered['vol'],
        name='Put Volume',
        text=put_data_filtered['vol_abbrev'],
        textposition='inside',
        marker=dict(color='red')
    ))

    # Update the layout
    fig.update_layout(
        title=f'Call vs Put Volume by Strike (Zoomed-in on {lower_bound:.2f}-{upper_bound:.2f})',
        titlefont=dict(size=20),
        xaxis_title='Strike',
        yaxis_title='Volume',
        barmode='group',
        bargap=0.15,
        bargroupgap=0.1,
        template='plotly_dark'
    )

    # Save as HTML
    fig.write_html('/mnt/data/call_vs_put_volume_zoomed.html')
    fig.write_html('calls_vs_put.html')
    # Get the absolute path of the HTML file
    file_path = os.path.abspath('calls_vs_put.html')
    # Selenium to capture image
    options = webdriver.ChromeOptions()


    options.add_argument('--headless')  # This argument enables headless mode
    options.add_argument('--ignore-certificate-errors')
    options.add_argument('--allow-running-insecure-content')
    options.add_argument('--ignore-ssl-errors=yes')
    options.add_argument('--ignore-certificate-errors-spki-list')
    options.add_argument("user-agent=Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/115.0.0.0 Safari/537.36")
    # Initialize WebDriver
    browser = webdriver.Chrome(options=options)


    browser.get(f'file://{file_path}')

    # Take screenshot
    screenshot_bytes = browser.get_screenshot_as_png()

    # Save the screenshot to a file (optional)
    with open('calls_vs_put.jpg', 'wb') as f:
        f.write(screenshot_bytes)

    # Close the browser
    browser.quit()

    return screenshot_bytes




async def volume_histogram(ticker):
    # Fetch data asynchronously
    options_df = await poly.get_option_chain_all(ticker)
    options_df = options_df.df
    
    # Check if the DataFrame is empty
    if options_df.empty:
        print("No data available for the given ticker.")
        return
    
    # Check if 'open interest' or 'vol' column exists, else return
    focus_column = 'vol' if 'OI' not in options_df.columns else 'OI'
    
    # Filter data for the specific ticker (optional)
    filtered_df = options_df[options_df['sym'] == ticker]
    
    # Check if the DataFrame is empty after filtering
    if filtered_df.empty:
        print(f"No data available for the ticker {ticker}.")
        return



    # Visualization 1: Histogram using Plotly with custom colors
    fig = px.histogram(
        filtered_df, 
        x="strike", 
        color="cp",
        nbins=30,
        title=f"Histogram of {focus_column.capitalize()} Across Different Strike Prices",
        labels={"strike": "Strike Price", "count": focus_column.capitalize()},
        color_discrete_map={"call": PLT_CANDLE_INCREASING, "put": PLT_CANDLE_DECREASING}
    )
    fig.update_layout(template=PLT_SCAT_STYLE_TEMPLATE)
   


async def plot_iv_surface(ticker):
    # Filter data for the specific ticker (optional)
    df = await poly.get_option_chain_all(ticker)
    df = df.df

    filtered_df = df[df['ticker'] == ticker]
    
    # Check if the DataFrame is empty after filtering
    if filtered_df.empty:
        print(f"No data available for the ticker {ticker}.")
        return
    
    # Convert the 'exp' column to datetime format for numerical conversion
    filtered_df['expiry'] = pd.to_datetime(filtered_df['expiry'])
    
    # Convert datetime to numerical format (e.g., days since the first date in the data)
    first_date = filtered_df['expiry'].min()
    filtered_df['expiry'] = (filtered_df['expiry'] - first_date).dt.days

    # Create the 3D surface plot
    surface_data = filtered_df.pivot_table(values='iv', index='strike', columns='expiry', aggfunc='mean', fill_value=0)
    
    fig = go.Figure(data=[go.Surface(z=surface_data.values, x=surface_data.columns, y=surface_data.index, colorscale='Viridis')])
    
    fig.update_layout(
        title=f'IV Surface for {ticker}',
        scene=dict(
            xaxis_title='Days to Expiration',
            yaxis_title='Strike Price',
            zaxis_title='Implied Volatility'
        ),
        template=PLT_SCAT_STYLE_TEMPLATE
    )
    

    fig.write_html('iv_surface.html')
    # Get the absolute path of the HTML file
    file_path = os.path.abspath('iv_surface.html')
    # Selenium to capture image
    options = webdriver.ChromeOptions()


    options.add_argument('--headless')  # This argument enables headless mode
    options.add_argument('--ignore-certificate-errors')
    options.add_argument('--allow-running-insecure-content')
    options.add_argument('--proxy-bypass-list=<-loopback>;www.sec.gov')
    options.add_argument('--ignore-ssl-errors=yes')
    options.add_argument('--ignore-certificate-errors-spki-list')
    options.add_argument("user-agent=Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/115.0.0.0 Safari/537.36")
    # Initialize WebDriver
    browser = webdriver.Chrome(options=options)


    browser.get(f'file://{file_path}')

    # Take screenshot
    screenshot_bytes = browser.get_screenshot_as_png()

    # Save the screenshot to a file (optional)
    with open('iv_surface.jpg', 'wb') as f:
        f.write(screenshot_bytes)

    # Close the browser
    browser.quit()
    return screenshot_bytes



# Function to plot consolidated gamma exposure across all strikes with custom styling and additional title information
async def plot_greek_exposure(ticker: str, greek:str):
    df = await poly.get_option_chain_all(ticker)
    df = df.df
    
    # Filter data for the specific ticker (optional)
    filtered_df = df[df['ticker'] == ticker]
    
    # Check if the DataFrame is empty after filtering
    if filtered_df.empty:
        print(f"No data available for the ticker {ticker}.")
        return
    
    # Group data by strike and sum up gamma
    greek_data = filtered_df.groupby(['strike'])[greek].sum().reset_index()
    
    # Find the strike with the most gamma
    max_greek_strike = greek_data.loc[greek_data[greek].idxmax()]['strike']
    

    # Create the plot with a color gradient based on gamma levels
    fig = px.bar(
        greek_data, 
        x='strike', 
        y=greek, 
        labels={'strike': 'Strike Price', greek: f'{greek} Exposure'},
        color=f'{greek}',  # Color gradient based on gamma levels
        color_continuous_scale='Viridis'
    )

    # Add a vertical line at the strike with maximum gamma
    fig.add_shape(
        go.layout.Shape(
            type="line",
            x0=max_greek_strike,
            x1=max_greek_strike,
            y0=0,
            y1=greek_data[greek].max(),
            line=dict(color="Gold", width=3, dash="dot")
        )
    )

    # Add annotations for gamma levels
    annotations = [
        dict(
            x=strike,
            y=gamma,
            xref="x",
            yref="y",
            text=f"{gamma:.2f}",
            showarrow=True,
            arrowhead=1,
            ax=0,
            ay=-40,
            font=dict(size=12, color='yellow' if strike == max_greek_strike else 'white')
        )
        for strike, gamma in zip(greek_data['strike'], greek_data[greek])
    ]

    # Update layout
    fig.update_layout(
        template='plotly_dark',
        title={
            'text': f"Consolidated {greek} Exposure for {ticker} (Max @ {max_greek_strike})",
            'y':0.95,
            'x':0.5,
            'xanchor': 'center',
            'yanchor': 'top'
        },
        font=dict(
            family="Courier New, monospace",
            size=14,
            color='white'
        ),
        annotations=annotations  # Add annotations to layout
    )





    fig.write_html(f'files/{greek}.html')
    # Get the absolute path of the HTML file
    file_path = os.path.abspath(f'files/{greek}.html')
    # Selenium to capture image
    options = webdriver.ChromeOptions()


    options.add_argument('--headless')  # This argument enables headless mode
    options.add_argument('--ignore-certificate-errors')
    options.add_argument('--allow-running-insecure-content')
    options.add_argument('--proxy-bypass-list=<-loopback>;www.sec.gov')
    options.add_argument('--ignore-ssl-errors=yes')
    options.add_argument('--ignore-certificate-errors-spki-list')
    options.add_argument("user-agent=Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/115.0.0.0 Safari/537.36")
    # Initialize WebDriver
    browser = webdriver.Chrome(options=options)


    browser.get(f'file://{file_path}')

    # Take screenshot
    screenshot_bytes = browser.get_screenshot_as_png()

    # Save the screenshot to a file (optional)
    with open(f'files/{greek}.jpg', 'wb') as f:
        f.write(screenshot_bytes)

    # Close the browser
    browser.quit()
    return screenshot_bytes



async def plot_upside_downside(df, ticker):
  
    # Create subplots
    fig = make_subplots(rows=2, cols=1, shared_xaxes=True,
                        subplot_titles=('MACD & Signal', 'RSI & Histogram'),
                        vertical_spacing=0.1)

    # Plot MACD and Signal on the first subplot
    fig.add_trace(go.Scatter(x=df['timestamp'], y=df['macd'], mode='lines', name='MACD'), row=1, col=1)
    fig.add_trace(go.Scatter(x=df['timestamp'], y=df['signal'], mode='lines', name='Signal'), row=1, col=1)
    
    # Plot RSI on the second subplot with yellow color
    fig.add_trace(go.Scatter(x=df['timestamp'], y=df['rsi'], mode='lines', name='RSI', line=dict(color='yellow')), row=2, col=1)
    
    # Plot Histogram as bar on the second subplot
    fig.add_trace(go.Bar(x=df['timestamp'], y=df['histogram'], name='Histogram'), row=2, col=1)

    # Add horizontal lines at y=30 and y=70 on the second subplot
    fig.add_trace(go.Scatter(x=df['timestamp'], y=[30]*len(df), mode='lines', name='RSI 30', line=dict(color='green', dash='dash')), row=2, col=1)
    fig.add_trace(go.Scatter(x=df['timestamp'], y=[70]*len(df), mode='lines', name='RSI 70', line=dict(color='red', dash='dash')), row=2, col=1)
    
    # Update layout
    fig.update_layout(
        title=f'MACD, Signal, RSI & Histogram for {ticker}',
        template=PLT_SCAT_STYLE_TEMPLATE,
        xaxis_title='Timestamp',
        yaxis1_title='Value',
        yaxis2_title='Value',
    )

    fig.write_html('upside_downside.html')
    file_path = os.path.abspath('upside_downside.html')
    # Get the absolute path of the HTML file

    # Selenium to capture image
    options = webdriver.ChromeOptions()


    options.add_argument('--headless')  # This argument enables headless mode
    options.add_argument('--ignore-certificate-errors')
    options.add_argument('--allow-running-insecure-content')
    options.add_argument('--proxy-bypass-list=<-loopback>;www.sec.gov')
    options.add_argument('--ignore-ssl-errors=yes')
    options.add_argument('--ignore-certificate-errors-spki-list')
    options.add_argument("user-agent=Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/115.0.0.0 Safari/537.36")
    # Initialize WebDriver
    browser = webdriver.Chrome(options=options)

    browser.get(f'file://{file_path}')

    # Take screenshot
    screenshot_bytes = browser.get_screenshot_as_png()

    # Save the screenshot to a file (optional)
    with open('chart.jpg', 'wb') as f:
        f.write(screenshot_bytes)

    # Close the browser
    browser.quit()
 
    return screenshot_bytes



async def plot_total_volume_by_expiry_async(df_calls: pd.DataFrame, df_puts: pd.DataFrame):




    
    fig = go.Figure(layout=go.Layout(
        template=PLT_CANDLE_STYLE_TEMPLATE,
        font=PLT_FONT,
    ))

    # Add bar for total call volume
    fig.add_trace(go.Bar(
        x=df_calls['Expiry'],
        y=df_calls['Total Call Volume'],
        name='Total Call Volume',
        marker_color='#00ACFF',  # PLT_CANDLE_INCREASING
        yaxis='y',
    ))

    # Add bar for total put volume
    fig.add_trace(go.Bar(
        x=df_puts['Expiry'],
        y=df_puts['Total Put Volume'],
        name='Total Put Volume',
        marker_color='#e4003a',  # PLT_CANDLE_DECREASING
        yaxis='y',
    ))

    # Add title, labels, and customize layout
    fig.update_layout(
        title='Total Call and Put Volume and Open Interest by Expiration Date',
        xaxis_title='Expiration Date',
        yaxis=dict(
            title='Total Volume',
            tickfont=dict(
                color="#fdc708"  # PLT_CANDLE_YAXIS_TEXT_COLOR
            )
        ),
        yaxis2=dict(
            title='Total Open Interest',
            overlaying='y',
            side='right',
        ),
    )


    fig.write_html('data/vol_calls_vs_puts.html')
    file_path = os.path.abspath('data/vol_calls_vs_puts.html')
    # Get the absolute path of the HTML file

    # Selenium to capture image
    options = webdriver.ChromeOptions()


    options.add_argument('--headless')  # This argument enables headless mode
    options.add_argument('--ignore-certificate-errors')
    options.add_argument('--allow-running-insecure-content')
    options.add_argument('--proxy-bypass-list=<-loopback>;www.sec.gov')
    options.add_argument('--ignore-ssl-errors=yes')
    options.add_argument('--ignore-certificate-errors-spki-list')
    options.add_argument("user-agent=Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/115.0.0.0 Safari/537.36")
    # Initialize WebDriver
    browser = webdriver.Chrome(options=options)

    browser.get(f'file://{file_path}')

    # Take screenshot
    screenshot_bytes = browser.get_screenshot_as_png()

    # Save the screenshot to a file (optional)
    with open('data/vol_calls_vs_puts.jpg', 'wb') as f:
        f.write(screenshot_bytes)

    # Close the browser
    browser.quit()
 
    return screenshot_bytes





# async def main():

#     df = pd.read_csv('all_options.csv')

#     await plot_gamma_exposure('AAPL')

# asyncio.run(main())


from polygon.websocket import WebSocketMessage,WebSocketClient, EquityAgg,EquityQuote,EquityTrade, Market
import pytz
from pytz import timezone
from discord_webhook import AsyncDiscordWebhook, DiscordEmbed
import asyncio
from apis.polygonio.mapping import OPTIONS_EXCHANGES, option_condition_desc_dict, option_condition_dict
from datetime import datetime
from math import isnan
from datetime import timezone
from .cfg import hex_colors
from apis.helpers import calculate_price_to_strike, get_human_readable_string
from list_sets.ticker_lists import most_active_tickers
from pytz import timezone
most_active_tickers = set(most_active_tickers)
batch_data_aggs = []
batch_data_trades = []



utc = timezone('UTC')
aware_datetime = utc.localize(datetime.utcnow())


async def handle_option_msg(msgs: WebSocketMessage, data_queue: asyncio.Queue, db_manager=None):
    global batch_data_aggs, batch_data_trades


    send_tasks = []  # List to hold all send tasks
    agg_message_data = {}
    symbol_state = {}
    for m in msgs:


        
        us_central = pytz.timezone('US/Central')
        utc = pytz.UTC
        symbol = get_human_readable_string(m.symbol)
        strike = symbol.get('strike_price')
        expiry = symbol.get('expiry_date')
        call_put = symbol.get('call_put')
        underlying_symbol = symbol.get('underlying_symbol')

        ticker_data = { 
            'underlying_symbol': underlying_symbol,
            'strike': strike,
            'call_put': call_put,
            'expiry': expiry,
            'option_symbol': m.symbol
        }
        # if db_manager is not None:
        #     asyncio.create_task(db_manager.insert_option_symbol(ticker_data['underlying_symbol'],ticker_data['call_put'], ticker_data['strike'], ticker_data['expiry'], ticker_data['option_symbol']))

        if underlying_symbol in most_active_tickers:
            if isinstance(m, EquityTrade):
                trade_message_data = {}
                trade_message_data['type'] = 'EquityOptionTrade'
                trade_message_data['expiry'] = expiry
                trade_message_data['expiry'] =  datetime.strptime(expiry, '%Y-%m-%d').date()
                trade_message_data['call_put'] = call_put
                trade_message_data['underlying_symbol'] = underlying_symbol
                trade_message_data['strike'] = strike
                

                trade_message_data['option_symbol'] = m.symbol
                trade_message_data['price'] = m.price
                trade_message_data['size'] = m.size
                

                
                trade_message_data['price_to_strike'] = calculate_price_to_strike(m.price, strike)


                timestamp = datetime.fromtimestamp(m.timestamp / 1000.0, tz=utc)
                naive_utc_datetime = aware_datetime.astimezone(timezone('UTC')).replace(tzinfo=None)
                trade_message_data['hour_of_day'] = timestamp.hour

                # Now, keep the timestamp in Eastern Time
                trade_message_data['timestamp'] = naive_utc_datetime

                trade_message_data['conditions'] = [option_condition_dict.get(condition) for condition in m.conditions] if m.conditions is not None else []
                trade_message_data['conditions'] = trade_message_data['conditions'][0]
                trade_message_data['weekday'] = timestamp.weekday()
                trade_message_data['exchange'] = OPTIONS_EXCHANGES.get(m.exchange)

                # Add price and volume-based features
                prev_state = symbol_state.get(m.symbol, {})
                trade_message_data['price_change'] = m.price - prev_state.get('prev_price', m.price)
                trade_message_data['volume_change'] = m.size - prev_state.get('prev_volume', m.size)

                # Update state
                symbol_state[m.symbol] = {'prev_price': m.price, 'prev_volume': m.size}
                # if db_manager is not None:
                #     asyncio.create_task(db_manager.save_structured_message(trade_message_data, "optiontrade"))

                
        

                await data_queue.put(trade_message_data)
            if isinstance(m, EquityAgg):
                

                agg_message_data['type'] = 'EquityOptionAgg'
                agg_message_data['underlying_symbol'] = underlying_symbol
                agg_message_data['strike'] = strike
                agg_message_data['expiry'] = expiry
                agg_message_data['expiry']  =datetime.strptime(expiry, '%Y-%m-%d').date()
                agg_message_data['call_put'] = call_put
                agg_message_data['option_symbol'] = m.symbol
                agg_message_data['total_volume'] = m.accumulated_volume
                agg_message_data['volume'] = m.volume
                agg_message_data['day_vwap'] = m.aggregate_vwap
                agg_message_data['official_open'] = m.official_open_price
                agg_message_data['last_price'] = m.close
                agg_message_data['open'] = m.open




                agg_message_data['price_diff'] = agg_message_data['last_price'] - agg_message_data['official_open']
                # Moneyness
                if not isnan(agg_message_data['strike']):
                    agg_message_data['moneyness'] = agg_message_data['last_price'] / agg_message_data['strike']
                
                # Price-VWAP Difference
                agg_message_data['price_vwap_diff'] = agg_message_data['last_price'] - agg_message_data['day_vwap']    
                # Price Percentage Change
                if not isnan(agg_message_data['official_open']):
                    agg_message_data['price_percent_change'] = ((agg_message_data['last_price'] - agg_message_data['official_open']) / agg_message_data['official_open']) * 100
                
                # Volume Percentage of Total
                if not isnan(agg_message_data['total_volume']):
                    agg_message_data['volume_percent_total'] = (agg_message_data['volume'] / agg_message_data['total_volume']) * 100
                
                # Volume-to-Price
                if not isnan(agg_message_data['last_price']):
                    agg_message_data['volume_to_price'] = agg_message_data['volume'] / agg_message_data['last_price']
                

                start_timestamp = datetime.fromtimestamp(m.start_timestamp / 1000.0, tz=utc)
                start_timestamp = start_timestamp.astimezone(us_central)
                agg_message_data['agg_timestamp'] = start_timestamp.strftime('%Y-%m-%d %H:%M:%S')
                agg_message_data['agg_timestamp'] = datetime.strptime(agg_message_data['agg_timestamp'], '%Y-%m-%d %H:%M:%S')
                end_timestamp = datetime.fromtimestamp(m.end_timestamp / 1_000_000_000.0, tz=utc)
                end_timestamp = end_timestamp.astimezone(us_central)
   
                volume = agg_message_data.get('volume', None)
                total_volume = agg_message_data.get('total_volume')
                ticker = agg_message_data.get('underlying_symbol')
                expiry = agg_message_data.get('expiry')
                call_put = agg_message_data.get('call_put')
                sym = agg_message_data.get('option_symbol')
                day_vwap = agg_message_data.get('day_vwap')
                official_open = agg_message_data.get('official_open')
                price = agg_message_data.get('price')
                open = agg_message_data.get('open')
                price_diff = agg_message_data.get('price_diff')
                moneyness = agg_message_data.get('moneyness')
                price_vwap_diff = agg_message_data.get('price_vwap_diff')
                price_percent_change = agg_message_data.get('price_percent_change')
                volume_percent_total = agg_message_data.get('volume_percent_total')
                volume_to_price = agg_message_data.get('volume_to_price')
                agg_timestamp = agg_message_data.get('agg_timestamp')


                await data_queue.put(agg_message_data)
                if volume > 500 and volume == total_volume:
                    hook = AsyncDiscordWebhook("https://discord.com/api/webhooks/1154856528688459827/lGUBqTHjwIlF0yo_HmgYSCY6UblLRxn0aXNuFnIe4HOlqFYJu-Hhk5v5hnVu-dVcRVVJ", content=f"<@375862240601047070>")
                    embed = DiscordEmbed(title=f'{ticker} {strike} {call_put} {expiry}', description=f'```py\nThis feed is returning tickers where the last trade for the contract == the total volume for that contract on the day.```', color=hex_colors['yellow'])
                    embed.add_embed_field(name=f"Feed:", value=f"> **Volume == Total Volume**", inline=False)
                    embed.add_embed_field(name=f"Day Stats:", value=f"> Open: **${official_open}**\n> Now: **${open}**\n> Price % Change: **{round(float(price_percent_change),2)}%**\n> Price Diff: **{price_diff}**\n> VWAP: **${day_vwap}**", inline=False)
                    embed.add_embed_field(name=f"Extras:", value=f"> Price/VWAP Diff: **{round(float(price_vwap_diff),2)}%**\n> Moneyness: **{round(float(moneyness),2)}%**")
                    embed.add_embed_field(name=f"Volume:", value=f"> Trade: **{float(volume):,}**\n> Total: **{total_volume}**\n> Volume % Total: **{round(float(volume_percent_total),2)}%**\n> Volume to Price: **{round(float(volume_to_price),2)}%**")
                    embed.set_timestamp()
                    embed.set_footer(text=f'{sym} | {agg_timestamp}')
                    hook.add_embed(embed)
                    asyncio.create_task(hook.execute())
                # if db_manager is not None:
                #     asyncio.create_task(db_manager.save_structured_message(agg_message_data, "optionagg"))


# async def process_and_store_data(data_list):
#     for data_dict in data_list:
#         # Parse each dictionary
#         break_even_price = data_dict.get('break_even_price', None)
#         session_change = data_dict.get('session.change', None)
#         session_change_percent = data_dict.get('session.change_percent', None)
#         session_early_trading_change = data_dict.get('session.early_trading_change', None)
#         session_early_trading_change_percent = data_dict.get('session.early_trading_change_percent', None)
#         session_close = data_dict.get('session.close', None)
#         session_high = data_dict.get('session.high', None)
#         session_low = data_dict.get('session.low', None)
#         session_open = data_dict.get('session.open', None)
#         session_volume = data_dict.get('session.volume', None)
#         session_previous_close = data_dict.get('session.previous_close', None)
        
#         contract_type = data_dict.get('details.contract_type', None)
#         exercise_style = data_dict.get('details.exercise_style', None)
#         expiration_date = data_dict.get('details.expiration_date', None)
#         strike_price = data_dict.get('details.strike_price', None)
        
#         delta = data_dict.get('greeks.delta', None)
#         gamma = data_dict.get('greeks.gamma', None)
#         theta = data_dict.get('greeks.theta', None)
#         vega = data_dict.get('greeks.vega', None)
        
#         implied_volatility = data_dict.get('implied_volatility', None)
        
#         last_ask = data_dict.get('last_quote.ask', None)
#         last_ask_size = data_dict.get('last_quote.ask_size', None)
#         last_ask_exchange = OPTIONS_EXCHANGES.get(data_dict.get('last_quote.ask_exchange'))
#         last_bid = data_dict.get('last_quote.bid', None)
#         last_bid_size = data_dict.get('last_quote.bid_size', None)
#         last_ask_exchange = OPTIONS_EXCHANGES.get(data_dict.get('last_quote.bid_exchange'))
#         midpoint = data_dict.get('last_quote.midpoint', None)


#         last_trade_timestamp = data_dict.get('last_trade.sip_timestamp', None)


#         if last_trade_timestamp is not None:
#             last_trade_timestamp = datetime.fromtimestamp(last_trade_timestamp / 1e9)
#         last_trade_conditions = [option_condition_dict.get(c) for c in last_trade_conditions] if last_trade_conditions is not None else []
#         last_trade_conditions = last_trade_conditions[0]
#         last_trade_exchange = OPTIONS_EXCHANGES.get(data_dict.get('last_trade'))
#         last_trade_price = data_dict.get('last_trade.price', None)
#         last_trade_size = data_dict.get('last_trade.size', None)

        
#         open_interest = data_dict.get('open_interest', None)
        
#         change_to_break_even = data_dict.get('underlying_asset.change_to_break_even', None)
#         asset_price = data_dict.get('underlying_asset.price', None)
#         asset_ticker = data_dict.get('underlying_asset.ticker', None)

#         name = data_dict.get('name', None)
#         ticker = data_dict.get('ticker', None)
#         await insert_into_database(
#             break_even_price=break_even_price,
#             session_change=session_change,
#             session_change_percent=session_change_percent,
#             session_early_trading_change=session_early_trading_change,
#             session_early_trading_change_percent=session_early_trading_change_percent,
#             session_close=session_close,
#             session_high=session_high,
#             session_low=session_low,
#             session_open=session_open,
#             session_volume=session_volume,
#             session_previous_close=session_previous_close,
#             contract_type=contract_type,
#             exercise_style=exercise_style,
#             expiration_date=expiration_date,
#             strike_price=strike_price,
#             delta=delta,
#             gamma=gamma,
#             theta=theta,
#             vega=vega,
#             implied_volatility=implied_volatility,
#             last_ask=last_ask,
#             last_ask_size=last_ask_size,
#             last_ask_exchange=last_ask_exchange,
#             last_bid=last_bid,
#             last_bid_size=last_bid_size,
#             last_bid_exchange=last_bid_exchange,
#             midpoint=midpoint,
#             last_trade_timestamp=last_trade_timestamp,
#             last_trade_conditions=last_trade_conditions,
#             last_trade_price=last_trade_price,
#             last_trade_size=last_trade_size,
#             last_trade_exchange=last_trade_exchange,
#             open_interest=open_interest,
#             change_to_break_even=change_to_break_even,
#             asset_price=asset_price,
#             asset_ticker=asset_ticker,
#             name=name,
#             ticker=ticker
#         )


hex_color_dict = {
    "red": "FF0000",
    "green": "00FF00",
    "blue": "0000FF",
    "yellow": "FFFF00",
    "purple": "800080",
    "cyan": "00FFFF",
    "magenta": "FF00FF",
    "lime": "00FF00",
    "pink": "FFC0CB",
    "teal": "008080",
    "lavender": "E6E6FA",
    "brown": "A52A2A",
    "beige": "F5F5DC",
    "maroon": "800000",
    "mint": "98FF98",
    "olive": "808000",
    "coral": "FF7F50",
    "navy": "000080",
    "grey": "808080",
    "white": "FFFFFF",
    "black": "000000",
    "orange": "FFA500",
    "indigo": "4B0082",
    "violet": "EE82EE",
    "gold": "FFD700",
    "silver": "C0C0C0",
    "rose": "FF007F",
    "emerald": "50C878",
    "ruby": "E0115F",
    "sapphire": "0F52BA",
    "amber": "FFBF00",
    "crimson": "DC143C",
    "darkgreen": "006400",
    "darkblue": "00008B",
    "darkred": "8B0000",
    "lightgreen": "90EE90",
    "lightblue": "ADD8E6",
    "lightyellow": "FFFFE0",
    "lightpink": "FFB6C1",
    "darkpurple": "800080",
    "lightpurple": "D8BFD8",
    "turquoise": "40E0D0",
    "tan": "D2B48C",
    "lightgrey": "D3D3D3",
    "darkgrey": "A9A9A9",
    "ivory": "FFFFF0",
    "khaki": "F0E68C",
    "lavenderblush": "FFF0F5",
    "lightcoral": "F08080",
    "salmon": "FA8072",
    "darksalmon": "E9967A",
    "lightsalmon": "FFA07A",
    "darkorange": "FF8C00",
    "linen": "FAF0E6",
    "peachpuff": "FFDAB9",
    "seashell": "FFF5EE",
    "snow": "FFFAFA",
    "wheat": "F5DEB3",
    "mistyrose": "FFE4E1",
    "azure": "F0FFFF",
    "aliceblue": "F0F8FF",
    "honeydew": "F0FFF0",
    "mintcream": "F5FFFA"
}


all_forex_pairs = {
'United States/Canada': 'USD/CAD',
'Japan/Euro': 'JPY/EUR',
'Denmark/Canada': 'DKK/CAD',
'Czech Republic/Japan': 'CZK/JPY',
'Euro/China': 'EUR/CNH',
'China/Singapore': 'CNY/SGD',
'Norway/United States': 'NOK/USD',
'Denmark/Mexico': 'DKK/MXN',
'Switzerland/South Africa': 'CHF/ZAR',
'Brazil/Japan': 'BRL/JPY',
'New Zealand/Indonesia': 'NZD/IDR',
'Bermuda/Canada': 'BMD/CAD',
'Silver/Turkey': 'XAG/TRY',
'Australia/South Africa': 'AUD/ZAR',
'Switzerland/Poland': 'CHF/PLN',
'Hong Kong/India': 'HKD/INR',
'Australia/New Zealand': 'AUD/NZD',
'South Africa/United Kingdom': 'ZAR/GBP',
'Malaysia/Australia': 'MYR/AUD',
'India/Taiwan': 'INR/TWD',
'South Korea/Switzerland': 'KRW/CHF',
'New Zealand/Switzerland': 'NZD/CHF',
'Israel/Japan': 'ILS/JPY',
'United States/South Africa': 'USD/ZAR',
'Hungary/Denmark': 'HUF/DKK',
'United States/China': 'USD/CNY',
'British Pound/Canada': 'GBP/CAD',
'Gold/Turkey': 'XAU/TRY',
'Mexican Peso/United States': 'MXN/USD',
'Hong Kong/Thailand': 'HKD/THB',
'Japanese Yen/British Pound': 'JPY/GBP',
'Euro/Singapore': 'EUR/SGD',
'Chilean Peso/British Pound': 'CLP/GBP',
'Colombian Peso/Mexican Peso': 'COP/MXN',
'Colombian Peso/United States': 'COP/USD',
'Brazilian Real/Mexican Peso': 'BRL/MXN',
'New Zealand/Czech Republic': 'NZD/CZK',
'South Korean Won/Australia': 'KRW/AUD',
'New Zealand/Hong Kong': 'NZD/HKD',
'British Pound/Costa Rican Colón': 'GBP/CRC',
'Turkish Lira/United States': 'TRY/USD',
'Silver/Russian Ruble': 'XAG/RUB',
'Silver/Euro': 'XAG/EUR',
'Australian Dollar/Poland': 'AUD/PLN',
'Australian Dollar/China': 'AUD/CNH',
'Euro/Australian Dollar': 'EUR/AUD',
'Japanese Yen/Taiwan': 'JPY/TWD',
'South African Rand/Thailand': 'ZAR/THB',
'New Zealand/Thailand': 'NZD/THB',
'British Pound/Indonesian Rupiah': 'GBP/IDR',
'Australian Dollar/Denmark': 'AUD/DKK',
'Silver/South African Rand': 'XAG/ZAR',
'Silver/Chinese Yuan': 'XAG/CNY',
'Australian Dollar/Hungary': 'AUD/HUF',
'Bulgarian Lev/Chinese Yuan': 'BGN/CNY',
'Singapore Dollar/British Pound': 'SGD/GBP',
'Hong Kong Dollar/Switzerland': 'HKD/CHF',
'Malaysian Ringgit/Singapore Dollar': 'MYR/SGD',
'Silver/United States Dollar': 'XAG/USD',
'United States Dollar/Taiwan Dollar': 'USD/TWD',
'Swedish Krona/Danish Krone': 'SEK/DKK',
'Bermuda/British Pound': 'BMD/GBP',
'British Pound/Euro': 'GBP/EUR',
'Moroccan Dirham/Australian Dollar': 'MAD/AUD',
'Israeli Shekel/Norwegian Krone': 'ILS/NOK',
'British Pound/Singapore Dollar': 'GBP/SGD',
'Singapore Dollar/South African Rand': 'SGD/ZAR',
'Indian Rupee/Singapore Dollar': 'INR/SGD',
'Japanese Yen/Indonesian Rupiah': 'JPY/IDR',
'Japanese Yen/Polish Złoty': 'JPY/PLN',
'British Pound/Danish Krone': 'GBP/DKK',
'Polish Złoty/Hungarian Forint': 'PLN/HUF',
'Turkish Lira/South African Rand': 'TRY/ZAR',
'South African Rand/Egyptian Pound': 'ZAR/EGP',
'Gold/South Korean Won': 'XAU/KRW',
'Egyptian Pound/Japanese Yen': 'EGP/JPY',
'South African Rand/Indian Rupee': 'ZAR/INR',
'Australian Dollar/Indian Rupee': 'AUD/INR',
'Gold/Mexican Peso': 'XAU/MXN',
'Japanese Yen/Canadian Dollar': 'JPY/CAD',
'Gold/British Pound': 'XAU/GBP',
'Danish Krone/Polish Złoty': 'DKK/PLN',
'South African Rand/Singapore Dollar': 'ZAR/SGD',
'Chinese Yuan/Swiss Franc': 'CNY/CHF',
'Chilean Peso/Australian Dollar': 'CLP/AUD',
'Japanese Yen/Indian Rupee': 'JPY/INR',
'Swiss Franc/Hong Kong Dollar': 'CHF/HKD',
'Hong Kong Dollar/Argentine Peso': 'HKD/ARS',
'Swiss Franc/Singapore Dollar': 'CHF/SGD',
'Czech Koruna/United States Dollar': 'CZK/USD',
'Chinese Yuan/South African Rand': 'CNY/ZAR',
'New Zealand/Euro': 'NZD/EUR',
'Australian Dollar/Swedish Krona': 'AUD/SEK',
'Swedish Krona/Polish Złoty': 'SEK/PLN',
'New Zealand/Indian Rupee': 'NZD/INR',
'South African Rand/Swiss Franc': 'ZAR/CHF',
'Indian Rupee/Swedish Krona': 'INR/SEK',
'Polish Złoty/British Pound': 'PLN/GBP',
'Danish Krone/Pakistani Rupee': 'DKK/PKR',
'Swedish Krona/British Pound': 'SEK/GBP',
'Canadian Dollar/Swiss Franc': 'CAD/CHF',
'New Zealand/Polish Złoty': 'NZD/PLN',
'South Korean Won/United States Dollar': 'KRW/USD',
'United States Dollar/Hong Kong Dollar': 'USD/HKD',
'Danish Krone/Thai Baht': 'DKK/THB',
'Gold/Thai Baht': 'XAU/THB',
'Philippine Peso/Danish Krone': 'PHP/DKK',
'Russian Ruble/Swiss Franc': 'RUB/CHF',
'Indian Rupee/Euro': 'INR/EUR',
'Argentine Peso/Euro': 'ARS/EUR',
'Polish Złoty/Czech Koruna': 'PLN/CZK',
'Norwegian Krone/Swedish Krona': 'NOK/SEK',
'Malaysian Ringgit/Swiss Franc': 'MYR/CHF',
'Canadian Dollar/United Arab Emirates Dirham': 'CAD/AED',
'Danish Krone/Swiss Franc': 'DKK/CHF',
'Silver/South Korean Won': 'XAG/KRW',
'Japanese Yen/Thai Baht': 'JPY/THB',
'Silver/Brazilian Real': 'XAG/BRL',
'Swiss Franc/Japanese Yen': 'CHF/JPY',
'Singapore Dollar/Indian Rupee': 'SGD/INR',
'Argentine Peso/Hong Kong Dollar': 'ARS/HKD',
'United States Dollar/Swedish Krona': 'USD/SEK',
'Cayman Islands Dollar/Canadian Dollar': 'KYD/CAD',
'British Pound/New Zealand Dollar': 'GBP/NZD',
'Taiwan Dollar/Danish Krone': 'TWD/DKK',
'Taiwan Dollar/Thai Baht': 'TWD/THB',
'Australian Dollar/Czech Koruna': 'AUD/CZK',
'Swiss Franc/United States Dollar': 'CHF/USD',
'United States Dollar/Danish Krone': 'USD/DKK',
'South African Rand/Philippine Peso': 'ZAR/PHP',
'Euro/Swiss Franc': 'EUR/CHF',
'United States Dollar/Icelandic Króna': 'USD/ISK',
'South African Rand/Botswana Pula': 'ZAR/BWP',
'Australian Dollar/Euro': 'AUD/EUR',
'Japanese Yen/Norwegian Krone': 'JPY/NOK',
'Swiss Franc/Romanian Leu': 'CHF/RON',
'Euro/British Pound': 'EUR/GBP',
'South Korean Won/Singapore Dollar': 'KRW/SGD',
'Peruvian Sol/Canadian Dollar': 'PEN/CAD',
'New Zealand Dollar/Japanese Yen': 'NZD/JPY',
'Malaysian Ringgit/South African Rand': 'MYR/ZAR',
'Swedish Krona/South African Rand': 'SEK/ZAR',
'Cypriot Pound/South African Rand': 'CYP/ZAR',
'Turkish Lira/Japanese Yen': 'TRY/JPY',
'British Pound/Israeli Shekel': 'GBP/ILS',
'South African Rand/New Zealand Dollar': 'ZAR/NZD',
'Turkish Lira/Polish Złoty': 'TRY/PLN',
'South African Rand/United States Dollar': 'ZAR/USD',
'Nigerian Naira/Japanese Yen': 'NGN/JPY',
'Silver/Hong Kong Dollar': 'XAG/HKD',
'Hong Kong Dollar/New Zealand Dollar': 'HKD/NZD',
'Swiss Franc/Swedish Krona': 'CHF/SEK',
'Japanese Yen/Australian Dollar': 'JPY/AUD',
'British Pound/Honduran Lempira': 'GBP/HNL',
'Singapore Dollar/Danish Krone': 'SGD/DKK',
'Malaysian Ringgit/Canadian Dollar': 'MYR/CAD',
'Czech Koruna/Canadian Dollar': 'CZK/CAD',
'Gold/Chinese Yuan': 'XAU/CNY',
'Gold/Canadian Dollar': 'XAU/CAD',
'Australian Dollar/British Pound': 'AUD/GBP',
'Swedish Krona/Swiss Franc': 'SEK/CHF',
'Australian Dollar/Hong Kong Dollar': 'AUD/HKD',
'South Korean Won/British Pound': 'KRW/GBP',
'Taiwan Dollar/Australian Dollar': 'TWD/AUD',
'Taiwan Dollar/Canadian Dollar': 'TWD/CAD',
'Malaysian Ringgit/Indonesian Rupiah': 'MYR/IDR',
'South Korean Won/Hong Kong Dollar': 'KRW/HKD',
'Taiwan Dollar/South African Rand': 'TWD/ZAR',
'New Zealand Dollar/Swedish Krona': 'NZD/SEK',
'Australian Dollar/Moroccan Dirham': 'AUD/MAD',
'New Zealand Dollar/United States Dollar': 'NZD/USD',
'Taiwan Dollar/Swedish Krona': 'TWD/SEK',
'United States Dollar/Indian Rupee': 'USD/INR',
'Norwegian Krone/Danish Krone': 'NOK/DKK',
'Czech Koruna/Mexican Peso': 'CZK/MXN',
'Thai Baht/Indonesian Rupiah': 'THB/IDR',
'United States Dollar/Egyptian Pound': 'USD/EGP',
'Canadian Dollar/Turkish Lira': 'CAD/TRY',
'Indonesian Rupiah/South Korean Won': 'IDR/KRW',
'South African Rand/Swedish Krona': 'ZAR/SEK',
'Israeli Shekel/Swiss Franc': 'ILS/CHF',
'British Pound/Icelandic Króna': 'GBP/ISK',
'Indian Rupee/Thai Baht': 'INR/THB',
'Danish Krone/Czech Koruna': 'DKK/CZK',
'Euro/Mexican Peso': 'EUR/MXN',
'Silver/Swiss Franc': 'XAG/GBP',
'Silver/Australian Dollar': 'XAG/AUD',
'Thai Baht/Taiwan Dollar': 'THB/TWD',
'South Korean Won/Canadian Dollar': 'KRW/CAD',
'New Zealand Dollar/Danish Krone': 'NZD/DKK',
'British Pound/Hungarian Forint': 'GBP/HUF',
'Chinese Yuan/United States Dollar': 'CNH/USD',
'Switzerland/Thailand': 'CHF/THB',
'Australia/South Korea': 'AUD/KRW',
'Czech Republic/Eurozone': 'CZK/EUR',
'Switzerland/Taiwan': 'CHF/TWD',
'Denmark/United Kingdom': 'DKK/GBP',
'Gold/Eurozone': 'XAU/EUR',
'Thai Baht/New Zealand Dollar': 'THB/NZD',
'Japanese Yen/Czech Koruna': 'JPY/CZK',
'Canadian Dollar/Hong Kong Dollar': 'CAD/HKD',
'British Pound/Norwegian Krone': 'GBP/NOK',
'South African Rand/Australian Dollar': 'ZAR/AUD',
'South African Rand/Czech Koruna': 'ZAR/CZK',
'South African Rand/Pakistani Rupee': 'ZAR/PKR',
'British Pound/Czech Koruna': 'GBP/CZK',
'Chinese Yuan/Japanese Yen': 'CNH/JPY',
'United States Dollar/Thai Baht': 'USD/THB',
'Danish Krone/New Zealand Dollar': 'DKK/NZD',
'British Pound/United States Dollar': 'GBP/USD',
'New Taiwan Dollar/Indonesian Rupiah': 'TWD/IDR',
'Singapore Dollar/Mexican Peso': 'SGD/MXN',
'Silver Ounce/Indian Rupee': 'XAG/INR',
'Norwegian Krone/British Pound': 'NOK/GBP',
'Canadian Dollar/Indonesian Rupiah': 'CAD/IDR',
'Argentine Peso/South African Rand': 'ARS/ZAR',
'Singapore Dollar/New Zealand Dollar': 'SGD/NZD',
'Brazilian Real/Argentine Peso': 'BRL/ARS',
'Malaysian Ringgit/Hong Kong Dollar': 'MYR/HKD',
'Gold/Hong Kong Dollar': 'XAU/HKD',
'Norwegian Krone/Eurozone': 'NOK/EUR',
'United States Dollar/Chinese Yuan': 'USD/CNH',
'Singapore Dollar/Swedish Krona': 'SGD/SEK',
'New Zealand Dollar/South African Rand': 'NZD/ZAR',
'Hong Kong Dollar/Canadian Dollar': 'HKD/CAD',
'Swedish Krona/Czech Koruna': 'SEK/CZK',
'Danish Krone/United States Dollar': 'DKK/USD',
'Eastern Caribbean Dollar/British Pound': 'XCD/GBP',
'Australian Dollar/Turkish Lira': 'AUD/TRY',
'Hong Kong Dollar/Australian Dollar': 'HKD/AUD',
'Danish Krone/Eurozone': 'DKK/EUR',
'Canadian Dollar/Israeli New Shekel': 'CAD/ILS',
'Japanese Yen/New Zealand Dollar': 'JPY/NZD',
'Hong Kong Dollar/Danish Krone': 'HKD/DKK',
'Swiss Franc/Indonesian Rupiah': 'CHF/IDR',
'Mexican Peso/Singapore Dollar': 'MXN/SGD',
'British Pound/South African Rand': 'GBP/ZAR',
'Canadian Dollar/Czech Koruna': 'CAD/CZK',
'United States Dollar/Norwegian Krone': 'USD/NOK',
'Japanese Yen/Israeli New Shekel': 'JPY/ILS',
'Gold/United States Dollar': 'XAU/USD',
'New Zealand Dollar/Canadian Dollar': 'NZD/CAD',
'Argentine Peso/Swiss Franc': 'ARS/CHF',
'Canadian Dollar/Japanese Yen': 'CAD/JPY',
'South African Rand/Cypriot Pound': 'ZAR/CYP',
'Indonesian Rupiah/British Pound': 'IDR/GBP',
'British Pound/South Korean Won': 'GBP/KRW',
'Indonesian Rupiah/South African Rand': 'IDR/ZAR',
'South African Rand/Japanese Yen': 'ZAR/JPY',
'Romanian Leu/United States Dollar': 'RON/USD',
'Hong Kong Dollar/New Taiwan Dollar': 'HKD/TWD',
'South African Rand/Canadian Dollar': 'ZAR/CAD',
'Hungarian Forint/Polish Zloty': 'HUF/PLN',
'New Taiwan Dollar/Singapore Dollar': 'TWD/SGD',
'Euro/Philippine Peso': 'EUR/PHP',
'Gold/Australian Dollar': 'XAU/AUD',
'Euro/Norwegian Krone': 'EUR/NOK',
'Hong Kong Dollar/Singapore Dollar': 'HKD/SGD',
'Singapore Dollar/Norwegian Krone': 'SGD/NOK',
'South Korean Won/South African Rand': 'KRW/ZAR',
'United States Dollar/Australian Dollar': 'USD/AUD',
'Swiss Franc/Hungarian Forint': 'CHF/HUF',
'Bulgarian Lev/Canadian Dollar': 'BGN/CAD',
'Singapore Dollar/Australian Dollar': 'SGD/AUD',
'Euro/Turkish Lira': 'EUR/TRY',
'Canadian Dollar/Norwegian Krone': 'CAD/NOK',
'Canadian Dollar/New Zealand Dollar': 'CAD/NZD',
'South African Rand/Norwegian Krone': 'ZAR/NOK',
'Chilean Peso/Swiss Franc': 'CLP/CHF',
'South Korean Won/Brazilian Real': 'KRW/BRL',
'Thai Baht/South Korean Won': 'THB/KRW',
'Mexican Peso/Australian Dollar': 'MXN/AUD',
'Silver Ounce/Canadian Dollar': 'XAG/CAD',
'British Pound/Chinese Yuan': 'GBP/CNH',
'Norwegian Krone/Indian Rupee': 'NOK/INR',
'Canadian Dollar/Indonesian Rupiah': 'CAD/IDR',
'Argentine Peso/South African Rand': 'ARS/ZAR',
'Singapore Dollar/New Zealand Dollar': 'SGD/NZD',
'Brazilian Real/Argentine Peso': 'BRL/ARS',
'Malaysian Ringgit/Hong Kong Dollar': 'MYR/HKD',
'Gold/Hong Kong Dollar': 'XAU/HKD',
'Norwegian Krone/Eurozone': 'NOK/EUR',
'United States Dollar/Chinese Yuan': 'USD/CNH',
'Singapore Dollar/Swedish Krona': 'SGD/SEK',
'New Zealand Dollar/South African Rand': 'NZD/ZAR',
'Hong Kong Dollar/Canadian Dollar': 'HKD/CAD',
'Swedish Krona/Czech Koruna': 'SEK/CZK',
'Danish Krone/United States Dollar': 'DKK/USD',
'Eastern Caribbean Dollar/British Pound': 'XCD/GBP',
'Australian Dollar/Turkish Lira': 'AUD/TRY',
'Hong Kong Dollar/Australian Dollar': 'HKD/AUD',
'Danish Krone/Eurozone': 'DKK/EUR',
'Canadian Dollar/Israeli New Shekel': 'CAD/ILS',
'Japanese Yen/New Zealand Dollar': 'JPY/NZD',
'Hong Kong Dollar/Danish Krone': 'HKD/DKK',
'Swiss Franc/Indonesian Rupiah': 'CHF/IDR',
'Mexican Peso/Singapore Dollar': 'MXN/SGD',
'British Pound/South African Rand': 'GBP/ZAR',
'Canadian Dollar/Czech Koruna': 'CAD/CZK',
'United States Dollar/Norwegian Krone': 'USD/NOK',
'Japanese Yen/Israeli New Shekel': 'JPY/ILS',
'Gold/United States Dollar': 'XAU/USD',
'New Zealand Dollar/Canadian Dollar': 'NZD/CAD',
'Argentine Peso/Swiss Franc': 'ARS/CHF',
'Canadian Dollar/Japanese Yen': 'CAD/JPY',
'South African Rand/Cypriot Pound': 'ZAR/CYP',
'Indonesian Rupiah/British Pound': 'IDR/GBP',
'British Pound/South Korean Won': 'GBP/KRW',
'Indonesian Rupiah/South African Rand': 'IDR/ZAR',
'South African Rand/Japanese Yen': 'ZAR/JPY',
'Romanian Leu/United States Dollar': 'RON/USD',
'Hong Kong Dollar/New Taiwan Dollar': 'HKD/TWD',
'South African Rand/Canadian Dollar': 'ZAR/CAD',
'Hungarian Forint/Polish Zloty': 'HUF/PLN',
'New Taiwan Dollar/Singapore Dollar': 'TWD/SGD',
'Euro/Philippine Peso': 'EUR/PHP',
'Gold/Australian Dollar': 'XAU/AUD',
'Euro/Norwegian Krone': 'EUR/NOK',
'Hong Kong Dollar/Singapore Dollar': 'HKD/SGD',
'Singapore Dollar/Norwegian Krone': 'SGD/NOK',
'South Korean Won/South African Rand': 'KRW/ZAR',
'United States Dollar/Australian Dollar': 'USD/AUD',
'Swiss Franc/Hungarian Forint': 'CHF/HUF',
'Bulgarian Lev/Canadian Dollar': 'BGN/CAD',
'Singapore Dollar/Australian Dollar': 'SGD/AUD',
'Euro/Turkish Lira': 'EUR/TRY',
'Canadian Dollar/Norwegian Krone': 'CAD/NOK',
'Canadian Dollar/New Zealand Dollar': 'CAD/NZD',
'South African Rand/Norwegian Krone': 'ZAR/NOK',
'Chilean Peso/Swiss Franc': 'CLP/CHF',
'South Korean Won/Brazilian Real': 'KRW/BRL',
'Thai Baht/South Korean Won': 'THB/KRW',
'Mexican Peso/Australian Dollar': 'MXN/AUD',
'Silver Ounce/Canadian Dollar': 'XAG/CAD',
'British Pound/Chinese Yuan': 'GBP/CNH',
'Norwegian Krone/Indian Rupee': 'NOK/INR',
'Canadian Dollar/Indonesian Rupiah': 'CAD/IDR',
'Argentine Peso/South African Rand': 'ARS/ZAR',
'Singapore Dollar/New Zealand Dollar': 'SGD/NZD',
'Brazilian Real/Argentine Peso': 'BRL/ARS',
'Malaysian Ringgit/Hong Kong Dollar': 'MYR/HKD',
'Gold/Hong Kong Dollar': 'XAU/HKD',
'Norwegian Krone/Eurozone': 'NOK/EUR',
'United States Dollar/Chinese Yuan': 'USD/CNH',
'Singapore Dollar/Swedish Krona': 'SGD/SEK',
'New Zealand Dollar/South African Rand': 'NZD/ZAR',
'Hong Kong Dollar/Canadian Dollar': 'HKD/CAD',
'Swedish Krona/Czech Koruna': 'SEK/CZK',
'Danish Krone/United States Dollar': 'DKK/USD',
'Eastern Caribbean Dollar/British Pound': 'XCD/GBP',
'Australian Dollar/Turkish Lira': 'AUD/TRY',
'Hong Kong Dollar/Australian Dollar': 'HKD/AUD',
'Danish Krone/Eurozone': 'DKK/EUR',
'Canadian Dollar/Israeli New Shekel': 'CAD/ILS',
'Japanese Yen/New Zealand Dollar': 'JPY/NZD',
'Hong Kong Dollar/Danish Krone': 'HKD/DKK',
'Swiss Franc/Indonesian Rupiah': 'CHF/IDR',
'Mexican Peso/Singapore Dollar': 'MXN/SGD',
'British Pound/South African Rand': 'GBP/ZAR',
'Canadian Dollar/Czech Koruna': 'CAD/CZK',
'United States Dollar/Norwegian Krone': 'USD/NOK',
'Japanese Yen/Israeli New Shekel': 'JPY/ILS',
'Gold/United States Dollar': 'XAU/USD',
'New Zealand Dollar/Canadian Dollar': 'NZD/CAD',
'Argentine Peso/Swiss Franc': 'ARS/CHF',
'Canadian Dollar/Japanese Yen': 'CAD/JPY',
'South African Rand/Cypriot Pound': 'ZAR/CYP',
'Indonesian Rupiah/British Pound': 'IDR/GBP',
'British Pound/South Korean Won': 'GBP/KRW',
'Indonesian Rupiah/South African Rand': 'IDR/ZAR',
'South African Rand/Japanese Yen': 'ZAR/JPY',
'Romanian Leu/United States Dollar': 'RON/USD',
'Hong Kong Dollar/New Taiwan Dollar': 'HKD/TWD',
'South African Rand/Canadian Dollar': 'ZAR/CAD',
'Hungarian Forint/Polish Zloty': 'HUF/PLN',
'New Taiwan Dollar/Singapore Dollar': 'TWD/SGD',
'Euro/Philippine Peso': 'EUR/PHP',
'Gold/Australian Dollar': 'XAU/AUD',
'Euro/Norwegian Krone': 'EUR/NOK',
'Hong Kong Dollar/Singapore Dollar': 'HKD/SGD',
'Singapore Dollar/Norwegian Krone': 'SGD/NOK',
'South Korean Won/South African Rand': 'KRW/ZAR',
    'Indian Rupee/Swiss Franc': 'INR/CHF',
    'Swiss Franc/Australian Dollar': 'CHF/AUD',
    'Danish Krone/Swedish Krona': 'DKK/SEK',
    'United Arab Emirates Dirham/South African Rand': 'AED/ZAR',
    'New Taiwan Dollar/New Zealand Dollar': 'TWD/NZD',
    'Singapore Dollar/Argentine Peso': 'SGD/ARS',
    'Australian Dollar/Swiss Franc': 'AUD/CHF',
    'Swedish Krona/Israeli New Shekel': 'SEK/ILS',
    'Gold (XAU)/Argentine Peso': 'XAU/ARS',
    'Euro/Danish Krone': 'EUR/DKK',
    'Peruvian Sol/British Pound Sterling': 'PEN/GBP',
    'Singapore Dollar/Turkish Lira': 'SGD/TRY',
    'New Taiwan Dollar/Chinese Yuan': 'TWD/CNY',
    'British Pound Sterling/Chilean Peso': 'GBP/CLP',
    'Argentine Peso/Singapore Dollar': 'ARS/SGD',
    'Nigerian Naira/South African Rand': 'NGN/ZAR',
    'Czech Koruna/Swiss Franc': 'CZK/CHF',
    'Euro/Canadian Dollar': 'EUR/CAD',
    'Euro/Polish Złoty': 'EUR/PLN',
    'Nigerian Naira/British Pound Sterling': 'NGN/GBP',
    'Singapore Dollar/Hong Kong Dollar': 'SGD/HKD',
    'Danish Krone/Indian Rupee': 'DKK/INR',
    'Singapore Dollar/Euro': 'SGD/EUR',
    'Canadian Dollar/South Korean Won': 'CAD/KRW',
    'Australian Dollar/United States Dollar': 'AUD/USD',
    'Mexican Peso/Japanese Yen': 'MXN/JPY',
    'Brazilian Real/Swiss Franc': 'BRL/CHF',
    'Danish Krone/Hungarian Forint': 'DKK/HUF',
    'Chinese Yuan/United States Dollar': 'CNY/USD',
    'South African Rand/Euro': 'ZAR/EUR',
    'Euro/Czech Koruna': 'EUR/CZK',
    'Canadian Dollar/Australian Dollar': 'CAD/AUD',
    'Canadian Dollar/Kuwaiti Dinar': 'CAD/KWD',
    'New Zealand Dollar/Mexican Peso': 'NZD/MXN',
    'Nicaraguan Córdoba/British Pound Sterling': 'NIO/GBP',
    'South Korean Won/Swedish Krona': 'KRW/SEK',
    'Danish Krone/Hong Kong Dollar': 'DKK/HKD',
    'Canadian Dollar/New Taiwan Dollar': 'CAD/TWD',
    'Norwegian Krone/Polish Złoty': 'NOK/PLN',
    'New Taiwan Dollar/Swiss Franc': 'TWD/CHF',
    'New Zealand Dollar/British Pound Sterling': 'NZD/GBP',
    'Canadian Dollar/Danish Krone': 'CAD/DKK',
    'Canadian Dollar/Thai Baht': 'CAD/THB',
    'CFP Franc/British Pound Sterling': 'XPF/GBP',
    'Australian Dollar/Singapore Dollar': 'AUD/SGD',
    'Indonesian Rupiah/Indian Rupee': 'IDR/INR',
    'Polish Złoty/Mexican Peso': 'PLN/MXN',
    'British Pound Sterling/Mexican Peso': 'GBP/MXN',
    'Thai Baht/Swiss Franc': 'THB/CHF',
    'Euro/New Taiwan Dollar': 'EUR/TWD',
    'Australian Dollar/Japanese Yen': 'AUD/JPY',
    'Danish Krone/Norwegian Krone': 'DKK/NOK',
    'South African Rand/Hong Kong Dollar': 'ZAR/HKD',
    'British Pound Sterling/Indian Rupee': 'GBP/INR',
    'Egyptian Pound/Euro': 'EGP/EUR',
    'Gold (XAU)/Indian Rupee': 'XAU/INR',
    'New Taiwan Dollar/Hong Kong Dollar': 'TWD/HKD',
    'Argentine Peso/Brazilian Real': 'ARS/BRL',
    'United States Dollar/Singapore Dollar': 'USD/SGD',
    'South Korean Won/Chinese Yuan': 'KRW/CNY',
    'Costa Rican Colón/British Pound Sterling': 'CRC/GBP',
    'Mexican Peso/South African Rand': 'MXN/ZAR',
    'Thai Baht/Hong Kong Dollar': 'THB/HKD',
    'British Pound Sterling/East Caribbean Dollar': 'GBP/XCD',
    'Gold (XAU)/Japanese Yen': 'XAU/JPY',
    'Canadian Dollar/Euro': 'CAD/EUR',
    'Pakistani Rupee/South African Rand': 'PKR/ZAR',
    'Mexican Peso/Polish Złoty': 'MXN/PLN',
    'Australian Dollar/Bulgarian Lev': 'AUD/BGN',
    'Indian Rupee/United States Dollar': 'INR/USD',
    'South African Rand/New Taiwan Dollar': 'ZAR/TWD',
    'New Zealand Dollar/Australian Dollar': 'NZD/AUD',
    'Argentine Peso/Australian Dollar': 'ARS/AUD',
    'Qatari Rial/South African Rand': 'QAR/ZAR',
    'Hungarian Forint/Canadian Dollar': 'HUF/CAD',
    'Mexican Peso/Canadian Dollar': 'MXN/CAD',
    'Swedish Krona/Australian Dollar': 'SEK/AUD',
    'South African Rand/Romanian Leu': 'ZAR/RON',
    'Malaysian Ringgit/Indian Rupee': 'MYR/INR',
    'Honduran Lempira/British Pound Sterling': 'HNL/GBP',
    'Polish Złoty/Japanese Yen': 'PLN/JPY',
    'United States Dollar/Euro': 'USD/EUR',
    'Singapore Dollar/Indonesian Rupiah': 'SGD/IDR',
    'Gold (XAU)/Saudi Riyal': 'XAU/SAR',
    'Israeli New Shekel/Australian Dollar': 'ILS/AUD',
    'Canadian Dollar/British Pound Sterling': 'CAD/GBP',
    'United States Dollar/British Pound Sterling': 'USD/GBP',
    'Thai Baht/Singapore Dollar': 'THB/SGD',
    'Indian Rupee/South Korean Won': 'INR/KRW',
    'Indian Rupee/New Zealand Dollar': 'INR/NZD',
    'Indian Rupee/Australian Dollar': 'INR/AUD',
    'British Pound Sterling/Bermudian Dollar': 'GBP/BMD',
    'Australian Dollar/Philippine Peso': 'AUD/PHP',
    'Hungarian Forint/South African Rand': 'HUF/ZAR',
    'United States Dollar/Philippine Peso': 'USD/PHP',
    'Swedish Krona/Norwegian Krone': 'SEK/NOK',
    'Chinese Yuan/British Pound Sterling': 'CNY/GBP',
    'Canadian Dollar/Hungarian Forint': 'CAD/HUF',
    'Icelandic Króna/Swiss Franc': 'ISK/CHF',
    'Bulgarian Lev/Australian Dollar': 'BGN/AUD',
    'Indian Rupee/Canadian Dollar': 'INR/CAD',
    'Euro/New Zealand Dollar': 'EUR/NZD',
    'Thai Baht/Canadian Dollar': 'THB/CAD',
    'Singapore Dollar/South Korean Won': 'SGD/KRW',
    'Swiss Franc/Euro': 'CHF/EUR',
    'Gold (XAU)/South African Rand': 'XAU/ZAR',
    'Australian Dollar/Chilean Peso': 'AUD/CLP',
    'Swiss Franc/Chilean Peso': 'CHF/CLP',
    'Vietnamese Đồng/Euro': 'VND/EUR',
    'Swiss Franc/Pakistani Rupee': 'CHF/PKR',
    'Swiss Franc/South Korean Won': 'CHF/KRW',
    'Argentine Peso/Japanese Yen': 'ARS/JPY',
    'Sri Lankan Rupee/South African Rand': 'LKR/ZAR',
    'Japanese Yen/South African Rand': 'JPY/ZAR',
    'Thai Baht/South African Rand': 'THB/ZAR',
    'Australian Dollar/Israeli New Shekel': 'AUD/ILS',
    'Swedish Krona/Euro': 'SEK/EUR',
    'South African Rand/Indonesian Rupiah': 'ZAR/IDR',
    'United States Dollar/Swiss Franc': 'USD/CHF',
    'Australian Dollar/Thai Baht': 'AUD/THB',
    'Euro/Argentine Peso': 'EUR/ARS',
    'South African Rand/Argentine Peso': 'ZAR/ARS',
    'Norwegian Krone/Swiss Franc': 'NOK/CHF',
    'New Taiwan Dollar/South Korean Won': 'TWD/KRW',
    'Singapore Dollar/Canadian Dollar': 'SGD/CAD',
    'Canadian Dollar/Singapore Dollar': 'CAD/SGD',
    'British Pound Sterling/Polish Złoty': 'GBP/PLN',
    'Vietnamese Đồng/Canadian Dollar': 'VND/CAD',
    'Canadian Dollar/Mexican Peso': 'CAD/MXN',
    'Polish Złoty/Swedish Krona': 'PLN/SEK',
    'British Pound Sterling/Argentine Peso': 'GBP/ARS',
    'Chinese Yuan/Canadian Dollar': 'CNY/CAD',
    'British Pound Sterling/Australian Dollar': 'GBP/AUD',
    'Thai Baht/Indian Rupee': 'THB/INR',
    'Swiss Franc/British Pound Sterling': 'CHF/GBP',
    'Singapore Dollar/United States Dollar': 'SGD/USD',
    'Japanese Yen/United States Dollar': 'JPY/USD',
    'Peruvian Sol/Argentine Peso': 'PEN/ARS',
    'United States Dollar/Polish Złoty': 'USD/PLN',
    'Australian Dollar/Norwegian Krone': 'AUD/NOK',
    'South African Rand/Nigerian Naira': 'ZAR/NGN',
    'Canadian Dollar/Swedish Krona': 'CAD/SEK',
    'Swiss Franc/Danish Krone': 'CHF/DKK',
    'Polish Złoty/Euro': 'PLN/EUR',
    'Czech Koruna/Swedish Krona': 'CZK/SEK',
    'Canadian Dollar/Polish Złoty': 'CAD/PLN',
    'United States Dollar/Czech Koruna': 'USD/CZK',
    'British Pound Sterling/Guatemalan Quetzal': 'GBP/GTQ',
    'Mexican Peso/Czech Koruna': 'MXN/CZK',
    'Chinese Yuan (Offshore)/Hong Kong Dollar': 'CNH/HKD',
    'Indonesian Rupiah/United States Dollar': 'IDR/USD',
    'United States Dollar/Mexican Peso': 'USD/MXN',
    'Malaysian Ringgit/New Taiwan Dollar': 'MYR/TWD',
    'Hungarian Forint/United States Dollar': 'HUF/USD',
    'Lithuanian Litas/Australian Dollar': 'LTL/AUD',
    'Euro/Indian Rupee': 'EUR/INR',
    'Turkish Lira/Singapore Dollar': 'TRY/SGD',
    'Swedish Krona/Japanese Yen': 'SEK/JPY',
    'Canadian Dollar/Bulgarian Lev': 'CAD/BGN',
    'New Taiwan Dollar/Indian Rupee': 'TWD/INR',
    'Japanese Yen/Argentine Peso': 'JPY/ARS',
    'United States Dollar/New Zealand Dollar': 'USD/NZD',
    'British Pound Sterling/Egyptian Pound': 'GBP/EGP',
    'Swiss Franc/Turkish Lira': 'CHF/TRY',
    'British Pound Sterling/Philippine Peso': 'GBP/PHP',
    'Colombian Peso/Argentine Peso': 'COP/ARS',
    'Hungarian Forint/Swiss Franc': 'HUF/CHF',
    'Hungarian Forint/Japanese Yen': 'HUF/JPY',
    'British Pound Sterling/Nicaraguan Córdoba': 'GBP/NIO',
    'Australian Dollar/New Taiwan Dollar': 'AUD/TWD',
    'Australian Dollar/Canadian Dollar': 'AUD/CAD',
    'Czech Koruna/Polish Złoty': 'CZK/PLN',
    'Mexican Peso/Euro': 'MXN/EUR',
    'Danish Krone/Australian Dollar': 'DKK/AUD',
    'Gold (XAU)/Brazilian Real': 'XAU/BRL',
    'British Pound Sterling/Japanese Yen': 'GBP/JPY',
    'United States Dollar/Japanese Yen': 'USD/JPY',
    'Thai Baht/Australian Dollar': 'THB/AUD',
    'Swiss Franc/Norwegian Krone': 'CHF/NOK',
    'Chinese Yuan/New Zealand Dollar': 'CNY/NZD',
    'Hong Kong Dollar/South Korean Won': 'HKD/KRW',
    'Guatemalan Quetzal/British Pound Sterling': 'GTQ/GBP',
    'New Zealand Dollar/Hungarian Forint': 'NZD/HUF',
    'Hong Kong Dollar/Indonesian Rupiah': 'HKD/IDR',
    'Tanzanian Shilling/South African Rand': 'TZS/ZAR',
    'British Pound Sterling / Turkish Lira': 'GBP/TRY',
    'Swiss Franc / Russian Ruble': 'CHF/RUB',
    'Polish Złoty / Danish Krone': 'PLN/DKK',
    'Australian Dollar / Argentine Peso': 'AUD/ARS',
    'Silver (XAG) / Japanese Yen': 'XAG/JPY',
    'Romanian Leu / Swiss Franc': 'RON/CHF',
    'British Pound Sterling / Hong Kong Dollar': 'GBP/HKD',
    'Mexican Peso / Hong Kong Dollar': 'MXN/HKD',
    'Thai Baht / United States Dollar': 'THB/USD',
    'Japanese Yen / Mexican Peso': 'JPY/MXN',
    'New Taiwan Dollar / United States Dollar': 'TWD/USD',
    'Canadian Dollar / United States Dollar': 'CAD/USD',
    'Danish Krone / Turkish Lira': 'DKK/TRY',
    'Romanian Leu / South African Rand': 'RON/ZAR',
    'British Pound Sterling / Nigerian Naira': 'GBP/NGN',
    'United States Dollar / Indonesian Rupiah': 'USD/IDR',
    'South African Rand / Turkish Lira': 'ZAR/TRY',
    'British Pound Sterling / Swiss Franc': 'GBP/CHF',
    'Turkish Lira / Swiss Franc': 'TRY/CHF',
    'British Pound Sterling / Peruvian Nuevo Sol': 'GBP/PEN',
    'Indonesian Rupiah / New Zealand Dollar': 'IDR/NZD',
    'Danish Krone / Japanese Yen': 'DKK/JPY',
    'Singapore Dollar / Swiss Franc': 'SGD/CHF',
    'Australian Dollar / Indonesian Rupiah': 'AUD/IDR',
    'Swiss Franc / Canadian Dollar': 'CHF/CAD',
    'Hong Kong Dollar / Swedish Krona': 'HKD/SEK',
    'Norwegian Krone / South African Rand': 'NOK/ZAR',
    'Malaysian Ringgit / New Zealand Dollar': 'MYR/NZD',
    'Turkish Lira / Danish Krone': 'TRY/DKK',
    'Indian Rupee / Hong Kong Dollar': 'INR/HKD',
    'Danish Krone / Russian Ruble': 'DKK/RUB',
    'Singapore Dollar / Japanese Yen': 'SGD/JPY',
    'Euro / Japanese Yen': 'EUR/JPY',
    'Norwegian Krone / Israeli New Shekel': 'NOK/ILS',
    'Danish Krone / Singapore Dollar': 'DKK/SGD',
    'Philippine Peso / United States Dollar': 'PHP/USD',
    'Australian Dollar / Mexican Peso': 'AUD/MXN',
    'South African Rand / Danish Krone': 'ZAR/DKK',
    'United States Dollar / Hungarian Forint': 'USD/HUF',
    'Canadian Dollar / Cayman Islands Dollar': 'CAD/KYD',
    'Euro / United States Dollar': 'EUR/USD',
    'Chinese Yuan / Hong Kong Dollar': 'CNY/HKD',
    'South Korean Won / New Zealand Dollar': 'KRW/NZD',
    'Silver (XAG) / Mexican Peso': 'XAG/MXN',
    'Danish Krone / New Taiwan Dollar': 'DKK/TWD',
    'Swedish Krona / United States Dollar': 'SEK/USD',
    'Canadian Dollar / Argentine Peso': 'CAD/ARS',
    'Euro / South African Rand': 'EUR/ZAR',
    'Philippine Peso / Japanese Yen': 'PHP/JPY',
    'Polish Złoty / Norwegian Krone': 'PLN/NOK',
    'New Zealand Dollar / Singapore Dollar': 'NZD/SGD',
    'Euro / Hong Kong Dollar': 'EUR/HKD',
    'Euro / Egyptian Pound': 'EUR/EGP',
    'Japanese Yen / Swedish Krona': 'JPY/SEK',
    'Hong Kong Dollar / Mexican Peso': 'HKD/MXN',
    'Brazilian Real / Australian Dollar': 'BRL/AUD',
    'United States Dollar / Turkish Lira': 'USD/TRY',
    'Indian Rupee / South African Rand': 'INR/ZAR',
    'South Korean Won / Euro': 'KRW/EUR',
    'Canadian Dollar / Russian Ruble': 'CAD/RUB',
    'Hong Kong Dollar / Japanese Yen': 'HKD/JPY',
    'Chinese Yuan / Indian Rupee': 'CNY/INR',
    'Israeli New Shekel / Swedish Krona': 'ILS/SEK',
    'Canadian Dollar / Indian Rupee': 'CAD/INR',
    'Mexican Peso / Danish Krone': 'MXN/DKK',
    'Canadian Dollar / Bermudian Dollar': 'CAD/BMD',
    'New Zealand Dollar / South Korean Won': 'NZD/KRW',
    'New Zealand Dollar / New Taiwan Dollar': 'NZD/TWD',
    'Hungarian Forint / Euro': 'HUF/EUR',
    'South African Rand / Hungarian Forint': 'ZAR/HUF',
    'Polish Złoty / United States Dollar': 'PLN/USD',
    'Danish Krone / Chinese Yuan': 'DKK/CNY',
    'Japanese Yen / Danish Krone': 'JPY/DKK',
    'South African Rand / South Korean Won': 'ZAR/KRW',
    'Egyptian Pound / United States Dollar': 'EGP/USD',
    'Euro / Hungarian Forint': 'EUR/HUF'}



crypto_currency_pairs = {
    'COVAL-USD': 'Circuits of Value to US Dollar',
    'BTT-USD': 'BitTorrent to US Dollar',
    'SEI-USD': 'SatoExchange to US Dollar',
    'CLV-USD': 'Clover Finance to US Dollar',
    'ALCX-USD': 'Alchemix to US Dollar',
    'AUCTION-USD': 'Bounce to US Dollar',
    'TRU-USD': 'TrueFi to US Dollar',
    'EOS-USD': 'EOS to US Dollar',
    'BNT-USD': 'Bancor to US Dollar',
    'SNT-USD': 'Status to US Dollar',
    'DASH-USD': 'Dash to US Dollar',
    'DNT-USD': 'district0x to US Dollar',
    'SPELL-USD': 'Spell to US Dollar',
    'LCX-USD': 'LCX to US Dollar',
    'SWFTC-USD': 'SwftCoin to US Dollar',
    'XRP-AUD': 'XRP to Australian Dollar',
    'INJ-USD': 'Injective Protocol to US Dollar',
    'REQ-USD': 'Request to US Dollar',
    'FIS-USD': 'Stafi to US Dollar',
    'ATOM-USD': 'Cosmos to US Dollar',
    'VET-USD': 'VeChain to US Dollar',
    'FORTH-USD': 'Ampleforth Governance Token to US Dollar',
    'SAND-USD': 'Sandbox to US Dollar',
    'OGN-USD': 'Origin Protocol to US Dollar',
    'JST-USD': 'JUST to US Dollar',
    'IOT-USD': 'IOTA to US Dollar',
    'AVAX-USD': 'Avalanche to US Dollar',
    'SOL-USD': 'Solana to US Dollar',
    'HBAR-USD': 'Hedera Hashgraph to US Dollar',
    'LTC-USD': 'Litecoin to US Dollar',
    'BCH-EUR': 'Bitcoin Cash to Euro',
    'CRO-USD': 'Crypto.com Coin to US Dollar',
    'UST-USD': 'TerraUSD to US Dollar',
    'ZIL-USD': 'Zilliqa to US Dollar',
    'ICX-USD': 'ICON to US Dollar',
    'XVG-USD': 'Verge to US Dollar',
    'BTC-USD': 'Bitcoin to US Dollar',
    'CGLD-USD': 'Celo to US Dollar',
    'TVK-USD': 'Theta Fuel to US Dollar',
    'COMP-USD': 'Compound to US Dollar',
    'LYM-USD': 'Lympo to US Dollar',
    'RAI-USD': 'Rai Reflex Index to US Dollar',
    'STX-USD': 'Stacks to US Dollar',
    'ACH-USD': 'Alchemy Pay to US Dollar',
    'ETH-AUD': 'Ethereum to Australian Dollar',
    'ENJ-USD': 'Enjin Coin to US Dollar',
    'FARM-USD': 'Harvest Finance to US Dollar',
    'IMX-USD': 'Immutable X to US Dollar',
    'MLN-USD': 'Enzyme to US Dollar',
    'BADGER-USD': 'Badger DAO to US Dollar',
    'DOT-USD': 'Polkadot to US Dollar',
    'LTC-EUR': 'Litecoin to Euro',
    'ADA-USD': 'Cardano to US Dollar',
    'XMR-USD': 'Monero to US Dollar',
    'UTK-USD': 'Utrust to US Dollar',
    'NKN-USD': 'NKN to US Dollar',
    'XRP-BTC': 'XRP to Bitcoin',
    'C98-USD': 'Coin98 to US Dollar',
    'NEAR-USD': 'NEAR Protocol to US Dollar',
    'CSM-USD': 'Cryptosol to US Dollar',
    'FTM-USD': 'Fantom to US Dollar',
    'CVX-USD': 'Convex Finance to US Dollar',
    'RLC-USD': 'iExec RLC to US Dollar',
    'SHIB-USD': 'Shiba Inu to US Dollar',
    'NEO-USD': 'NEO to US Dollar',
    '1INCH-USD': '1inch to US Dollar',
    'ARPA-USD': 'ARPA Chain to US Dollar',
    'XLM-USD': 'Stellar Lumens to US Dollar',
    'MKR-USD': 'Maker to US Dollar',
    'GNO-USD': 'Gnosis to US Dollar',
    'IOTX-USD': 'IoTeX to US Dollar',
    'KNC-USD': 'Kyber Network to US Dollar',
    'APE-USD': 'ApeCoin to US Dollar',
    'CTSI-USD': 'Cartesi to US Dollar',
    'ETH-BTC': 'Ethereum to Bitcoin',
    'GLM-USD': 'Golem to US Dollar',
    'LUNA-USD': 'Terra to US Dollar',
    'USDT-USD': 'Tether to US Dollar',
    'LTC-BTC': 'Litecoin to Bitcoin',
    'BOND-USD': 'BarnBridge to US Dollar',
    'RARI-USD': 'Rarible to US Dollar',
    'COTI-USD': 'COTI to US Dollar',
    'GRT-USD': 'The Graph to US Dollar',
    'STORJ-USD': 'Storj to US Dollar',
    'GUSD-USD': 'Gemini Dollar to US Dollar',
    'UNI-USD': 'Uniswap to US Dollar',
    'UMA-USD': 'UMA to US Dollar',
    'OCEAN-USD': 'Ocean Protocol to US Dollar',
    'FIL-USD': 'Filecoin to US Dollar',
    'BTC-GBP': 'Bitcoin to British Pound',
    'USDC-USD': 'USD Coin to US Dollar',
    'API3-USD': 'API3 to US Dollar',
    'QNT-USD': 'Quant to US Dollar',
    'ALGO-USD': 'Algorand to US Dollar',
    'DYDX-USD': 'dYdX to US Dollar',
    'BTC-EUR': 'Bitcoin to Euro',
    'SUSHI-USD': 'SushiSwap to US Dollar',
    'BCH-USD': 'Bitcoin Cash to US Dollar',
    'CRV-USD': 'Curve DAO Token to US Dollar',
    'RUNE-USD': 'THORChain to US Dollar',
    'ZEC-USD': 'Zcash to US Dollar',
    'XRP-USD': 'XRP to US Dollar',
    'PAX-USD': 'Paxos Standard to US Dollar',
    'ZRX-USD': '0x to US Dollar',
    'ETC-USD': 'Ethereum Classic to US Dollar',
    'AMP-USD': 'Amp to US Dollar',
    'SUKU-USD': 'SUKU to US Dollar',
    'MANA-USD': 'Decentraland to US Dollar',
    'BCH-GBP': 'Bitcoin Cash to British Pound',
    'AVT-USD': 'Aventus to US Dollar',
    'QTF-USD': 'Quickswap to US Dollar',
    'ALICE-USD': 'MyNeighborAlice to US Dollar',
    'PERP-USD': 'Perpetual Protocol to US Dollar',
    'ROSE-USD': 'Oasis Network to US Dollar',
    'FET-USD': 'Fetch.ai to US Dollar',
    'TRX-USD': 'TRON to US Dollar',
    'NCT-USD': 'PolySwarm to US Dollar',
    'OXT-USD': 'Orchid to US Dollar',
    'BTRST-USD': 'Bitrue Coin to US Dollar',
    'PNK-USD': 'Kleros to US Dollar',
    'GTC-USD': 'Gitcoin to US Dollar',
    'TRAC-USD': 'OriginTrail to US Dollar',
    'PLA-USD': 'PlayDapp to US Dollar',
    'ZEN-USD': 'Horizen to US Dollar',
    'DSH-USD': 'Dashcoin to US Dollar',
    'AXS-USD': 'Axie Infinity to US Dollar',
    'SRM-USD': 'Serum to US Dollar',
    'RNDR-USD': 'Render Token to US Dollar',
    'XTZ-USD': 'Tezos to US Dollar',
    'SC-USD': 'Siacoin to US Dollar',
    'OMG-USD': 'OMG Network to US Dollar',
    'PAXG-USD': 'Paxos Gold to US Dollar',
    'FX-USD': 'Function X to US Dollar',
    'LRC-USD': 'Loopring to US Dollar',
    'ANKR-USD': 'Ankr to US Dollar',
    'ETH-USD': 'Ethereum to US Dollar',
    'KSM-USD': 'Kusama to US Dollar',
    'TRB-USD': 'Tellor to US Dollar',
    'NANO-USD': 'Nano to US Dollar',
    'CVC-USD': 'Civic to US Dollar',
    'QRDO-USD': 'Qredo to US Dollar',
    'XYO-USD': 'XYO Network to US Dollar',
    'WBTC-USD': 'Wrapped Bitcoin to US Dollar',
    'DOGE-USD': 'Dogecoin to US Dollar',
    'FCL-USD': 'Fractal to US Dollar',
    'GALA-USD': 'Gala to US Dollar',
    'JASMY-USD': 'JasmyCoin to US Dollar',
    'KAVA-USD': 'Kava to US Dollar',
    'TIA-USD': 'Tia Token to US Dollar',
    'BOSON-USD': 'Boson Protocol to US Dollar',
    'ICP-USD': 'Internet Computer to US Dollar',
    'WAVES-USD': 'Waves to US Dollar',
    'LINK-USD': 'Chainlink to US Dollar',
    'BOBA-USD': 'Boba Network to US Dollar',
    'DYDX-EUR': 'dYdX to Euro',
    'QTUM-USD': 'Qtum to US Dollar',
    'YFI-USD': 'yearn.finance to US Dollar',
    'ELA-USD': 'Elastos to US Dollar',
    'CHZ-USD': 'Chiliz to US Dollar',
    'BAND-USD': 'Band Protocol to US Dollar',
    'MASK-USD': 'Mask Network to US Dollar',
    'NEXO-USD': 'Nexo to US Dollar',
    'SEI-EUR': 'SatoExchange to Euro',
    'XRD-USD': 'Radix to US Dollar',
    'KEEP-USD': 'Keep3rV1 to US Dollar',
    'EGLD-USD': 'Elrond to US Dollar',
    'LPT-USD': 'Livepeer to US Dollar',
    'FOX-USD': 'ShapeShift FOX Token to US Dollar',
    'NMR-USD': 'Numeraire to US Dollar',
    'SGB-USD': 'Digital Silver to US Dollar',
    'DUSK-USD': 'Dusk Network to US Dollar',
    'BTC-AUD': 'Bitcoin to Australian Dollar',
    'FLOW-USD': 'Flow to US Dollar',
    'SNX-USD': 'Synthetix to US Dollar',
    'SKL-USD': 'Skale Network to US Dollar',
    'DAI-USD': 'Dai to US Dollar',
    'GHST-USD': 'Aavegotchi to US Dollar',
    'MATIC-USD': 'Polygon to US Dollar',
    'ORN-USD': 'Orion Protocol to US Dollar',
    'GYEN-USD': 'Digital Yen to US Dollar',
    'AAVE-USD': 'Aave to US Dollar',
    'UOS-USD': 'Ultra to US Dollar',
    'TLOS-USD': 'Telos to US Dollar',
    'ASM-USD': 'Assemble Protocol (ASM) to US Dollar',
    'BAL-USD': 'Balancer (BAL) to US Dollar',
    'KRL-USD': 'Kryll (KRL) to US Dollar',
    'BAT-USD': 'Basic Attention Token (BAT) to US Dollar'}


from apis.polygonio.async_polygon_sdk import Polygon

poly = Polygon()

import asyncio


async def ticker_snapshot():

    ticker_data = await poly.get_all_tickers(save_all_tickers=True)


    print(f"Number of tickers found: {len(ticker_data)}")

    print(ticker_data)

asyncio.run(ticker_snapshot())


import os
import re
from dotenv import load_dotenv
load_dotenv()
from disnake.ext import commands
import disnake
import pandas as pd
from datetime import datetime
from apis.webull.webull_options import WebullOptions
import disnake
from disnake import TextInputStyle
from tabulate import tabulate
from discord_.bot_menus.pagination import AlertMenus
from apis.helpers import chunk_string, human_readable
from apis.webull.modal import WebullModal, VolumeAnalysisModal
from apis.polygonio.polygon_database import PolygonDatabase
from discord_.bot_menus.pagination import AlertMenus
from typing import List
options = WebullOptions(connection_string=os.environ.get('WEBULL_OPTIONS'))




class DatabaseCOG(commands.Cog):
    def __init__(self, bot):
        self.bot=bot
        self.db = PolygonDatabase(user='postgres', database='fudstop')





    @commands.slash_command()
    async def database(self, inter):
        pass



    @database.sub_command()
    async def rsi(self, inter:disnake.AppCmdInter, status:str=commands.Param(choices=['oversold', 'overbought', "overbought', 'oversold"])):
        """Search the database for overbought/oversold tickers."""
        await inter.response.defer()
        query = f"""
        SELECT ticker, rsi_value, timespan, status from rsi where status in ('{status}') AND timespan in ('day','week') ORDER BY timespan ASC;
        """
        print(query)
        await self.db.connect()
        records = await self.db.fetch(query)
        df = pd.DataFrame(records)
        table = tabulate(df, headers='keys', tablefmt='fancy', showindex=False)
        
        chunks = chunk_string(table, 4000)
        embeds = []
        for chunk in chunks:
            embed = disnake.Embed(title=f"RSI Results | Database", description=f"```py\n{chunk}```", color=disnake.Colour.dark_gold())

            embed.add_field(name=f"Query:", value=f"> **{query}**")
            embeds.append(embed)

        await inter.edit_original_message(embed=embeds[0], view=AlertMenus(embeds))
            

    @database.sub_command()
    async def atm_options(self, inter:disnake.AppCmdInter, ticker:str):
        """Returns options within 10% of the current strike in both directions."""
        await inter.response.defer()
        ticker = ticker.upper()

        data, tickers = await self.db.atm_options(ticker=ticker)

        # Assuming tickers is a list of option symbols
        options = tickers  # Use all available tickers
        MAX_OPTIONS_PER_SELECT = 25
        MAX_SELECTS_PER_VIEW = 4
        # Your logic for pagination
        chunks = [options[i:i + MAX_OPTIONS_PER_SELECT] for i in range(0, len(options), MAX_OPTIONS_PER_SELECT)]
        grouped_chunks = [chunks[i:i + MAX_SELECTS_PER_VIEW] for i in range(0, len(chunks), MAX_SELECTS_PER_VIEW)]

        current_page = 0

        table = tabulate(data.sort_values('exp', ascending=True), headers='keys', tablefmt='fancy', showindex=False)

        chunks = chunk_string(table, 4000)
        
        embeds = [ ]
        for chunk in chunks:
            embed = disnake.Embed(title=f'Results for {ticker}', description=f"```py\n{chunk}```", color=disnake.Colour.dark_gold())
            embeds.append(embed)
        # In your atm_options function
        view = OptionsView(grouped_chunks, current_page)
        await inter.edit_original_message(view=view, embed=embeds[0])


    # @database.sub_command()
    # async def custom_filter(self, inter:disnake.AppCmdInter, ticker=None, call_put=None, strike_min=None, strike_max=None, 
    #             expiry_min=None, expiry_max=None,dte_min=None,dte_max=None,
    #             time_value_min=None, time_value_max=None, 
    #             intrinsic_value_min=None, intrinsic_value_max=None, 
    #             extrinsic_value_min=None, extrinsic_value_max=None, 
    #             vol_min=None, vol_max=None, oi_min=None,oi_max=None, 
    #             theta_min=None, theta_max=None, gamma_min=None,
    #             gamma_max=None, gamma_risk_min=None, gamma_risk_max=None, 
    #             delta_min=None, delta_max=None, delta_theta_ratio_min=None, 
    #             delta_theta_ratio_max=None, vega_min=None,
    #             vega_max=None, vega_impact_min=None, vega_impact_max=None):
    #     await inter.response.defer()
    #     await self.db.connect()

    #     # Base query string
    #     base_query = "SELECT * FROM opts WHERE "
    #     conditions = []

    #     # Check each parameter and append condition if not None
    #     if ticker is not None:
    #         conditions.append(f"ticker = '{ticker}'")
    #     if call_put is not None:
    #         conditions.append(f"call_put = '{call_put}'")
        
    #     # Add conditions for range parameters
    #     def add_range_condition(param_min, param_max, column_name):
    #         if param_min is not None:
    #             conditions.append(f"{column_name} >= {param_min}")
    #         if param_max is not None:
    #             conditions.append(f"{column_name} <= {param_max}")

    #     # Apply for all range parameters
    #     add_range_condition(strike_min, strike_max, "strike")
    #     add_range_condition(expiry_min, expiry_max, "expiry")
    #     add_range_condition(dte_min, dte_max, "dte")
    #     add_range_condition(time_value_min, time_value_max, "time_value")
    #     add_range_condition(intrinsic_value_min, intrinsic_value_max, "intrinsic_value")
    #     add_range_condition(extrinsic_value_min, extrinsic_value_max, "extrinsic_value")
    #     add_range_condition(vol_min, vol_max, "vol")
    #     add_range_condition(oi_min, oi_max, "oi")
    #     add_range_condition(theta_min, theta_max, "theta")
    #     add_range_condition(gamma_min, gamma_max, "gamma")
    #     add_range_condition(gamma_risk_min, gamma_risk_max, "gamma_risk")
    #     add_range_condition(delta_min, delta_max, "delta")
    #     add_range_condition(delta_theta_ratio_min, delta_theta_ratio_max, "delta_theta_ratio")
    #     add_range_condition(vega_min, vega_max, "vega")
    #     add_range_condition(vega_impact_min, vega_impact_max, "vega_impact")


    #     # Final query
    #     query = base_query + ' AND '.join(conditions) if conditions else "SELECT * FROM options"
    #     print(query)
    #     # Execute the query
    #     data = await self.db.fetch(query)
    #     df = pd.DataFrame(data).iloc[[0]]
    #     embed = disnake.Embed(title=f"Test", description=f"```py\n{df}```")
    #     await inter.edit_original_message(embed=embed)


    @database.sub_command()
    async def strategy(self, inter:disnake.AppCmdInter, strategy:str=commands.Param(choices=['low theta'])):
        """Choose a strategy to run for options - returns a spreadsheet."""
        await inter.response.defer()
        if strategy == 'low theta':
            data = await self.db.strategy_filter_theta()
            print(data)


            data.to_csv('data.csv')

            # table = tabulate(df_selected, headers='keys')

            # chunks = chunk_string(table, 4000)
            # embeds = []
            # for chunk in chunks:
            #     embed = disnake.Embed(title='Theta Stratey Results', description=f"```py\n{chunk}```", color=disnake.Colour.dark_gold())

            #     embed.add_field(name=f"Low Theta", value='```py\nThis strategy takes advantage of time on the clock as well as low-theta. This allows for an easily managed position and works in over-extended markets. [the current situation].```')

            #     embeds.append(embed)

            # await inter.edit_original_message(embed=embeds[0], view=AlertMenus(embeds))

            await inter.send(file=disnake.File('data.csv'))

cog = DatabaseCOG(bot=commands.Bot)


class OptionsView(disnake.ui.View):
    def __init__(self, grouped_chunks, current_page):
        super().__init__()
        self.grouped_chunks = grouped_chunks
        self.current_page = current_page
        # Add buttons and select menus

        for chunk in grouped_chunks[current_page]:
            options = [disnake.SelectOption(label=human_readable(ticker), description='Click me for data!') for ticker in chunk]
            self.select_menu = TickerSelect(options)
            self.add_item(self.select_menu)

    async def generate_view(self, option_chunks, current_page):
        return self(option_chunks, current_page)
    @disnake.ui.button(label="Previous", custom_id="prev_page")
    async def prev_button(self, button: disnake.ui.Button, interaction: disnake.MessageInteraction):
        self.current_page -= 1
        if self.current_page >= 0:
            # Update view for the previous page
            new_view = await self.generate_view(self.grouped_chunks, self.current_page)
            await interaction.response.edit_message(view=new_view)
        else:
            await interaction.response.send_message("This is the first page.", ephemeral=True)


    @disnake.ui.button(label="Next", custom_id="next_page")
    async def next_button(self, button: disnake.ui.Button, interaction: disnake.MessageInteraction):
        self.current_page += 1
        if self.current_page < len(self.grouped_chunks):
            # Update view for the next page
            new_view = await self.generate_view(self.grouped_chunks, self.current_page)
            await interaction.response.edit_message(view=new_view)
        else:
            await interaction.response.send_message("This is the last page.", ephemeral=True)




class TickerSelect(disnake.ui.Select):
    def __init__(self, options, **kwargs):
        super().__init__(placeholder='Choose an option', options=options,min_values=1,max_values=25, **kwargs)

    async def callback(self, interaction: disnake.MessageInteraction):
        await interaction.response.defer(ephemeral=True)
        await cog.db.connect()


        embeds = []
        for value in self._selected_values:
            parts = value.split(' ')
            ticker = parts[0]
            strike = parts[1].replace('$','').replace('.00','')
            call_put = parts[2].lower()
            expiry = parts[4]      

            query = f"""SELECT distinct * FROM opts WHERE ticker = '{ticker}' AND strike = {strike} AND cp = '{call_put}' AND expiry = '{expiry}';"""
            print(query)
            records = await cog.db.fetch(query)
            for record in records:
                dte = record['dte']
                time_value = record['time_value']
                moneyness = record['moneyness']
                liquidity_score = record['liquidity_score']
                theta = record['theta']
                theta_decay_rate = record['theta_decay_rate']
                delta = record['delta']
                delta_theta_ratio = record['delta_theta_ratio']
                gamma = record['gamma']
                gamma_risk = record['gamma_risk']
                vega = record['vega']
                vega_impact = record['vega_impact']
                timestamp = record['timestamp']
                oi = record['oi']
                open_price = record['open']
                high = record['high']
                low = record['low']
                close = record['close']
                intrinstic_value = record['intrinstic_value']
                extrinsic_value = record['extrinsic_value']
                leverage_ratio = record['leverage_ratio']
                vwap = record['vwap']
                conditions = record['conditions']
                price = record['price']
                trade_size = record['trade_size']
                exchange = record['exchange']
                ask = record['ask']
                bid = record['bid']
                spread = record['spread']
                spread_pct = record['spread_pct']
                iv = record['iv']
                bid_size = record['bid_size']
                ask_size = record['ask_size']
                vol = record['vol']
                mid = record['mid']
                change_to_breakeven = record['change_to_breakeven']
                underlying_price = record['underlying_price']
                ticker = record['ticker']
                return_on_risk = record['return_on_risk']
                velocity = record['velocity']
                sensitivity = record['sensitivity']
                greeks_balance = record['greeks_balance']
                opp = record['opp']
                insertion_timestamp = record['insertion_timestamp']
                embed = disnake.Embed(title=f"Selected Option: {record['ticker']} ${record['strike']} {record['cp']} {record['expiry']}", description=f"```py\nDTE: {dte}\n> Time Value: ${time_value}\n> Intrinsic value: ${intrinstic_value}\n> Extrinsic Value: ${extrinsic_value}\n\n> Open: ${open_price}\n> High: ${high}\n> Low: ${low}\n> Close: ${close}```", color=disnake.Colour.dark_teal())
                embed.add_field(name=f"Volume & OI", value=f"> **{vol}** // **{oi}**")
                embed.add_field(name=f"Delta:", value=f"> Value: **{delta}**\n> Delta/Theta Ratio: {delta_theta_ratio}**")
                embed.add_field(name=f"Vega:", value=f"> Value: **{vega}**\n> Impact: **{vega}**")
                embed.add_field(name=f"Gamma:", value=f"> Value: **{gamma}**\n> Risk: **{gamma_risk}**")
                embed.add_field(name="Theta:", value=f"> Value: **{theta}**\n> Decay Rate: **{theta_decay_rate}**")
                embed.add_field(name=f"IV:", value=f"> Value: **{round(float(iv)*100,2)}%**\n> Sensitivity: **{sensitivity}**\n> Velocity: **{velocity}**")
                embed.add_field(name=f"Bid/Ask/Spread:", value=f"> Bid: **${bid}**\n> Ask: **${ask}**\n> Spread: **{round(float(spread),2)}**\n> Spread Pct: **{spread_pct}%**")
                embed.add_field(name=f"Return/Risk:", value=f"> RoR: **{return_on_risk}**\n> ProfitPotential: **{opp}**\n> Greek Balance: **{greeks_balance}**")
                embed.add_field(name=f"Entry Cost:", value=f"> Mid: **${mid}**\n> VWAP: **${vwap}**\n> {ticker} Price: **${underlying_price}**")
                embeds.append(embed)
            await interaction.edit_original_message(embed=embeds[0], view=AlertMenus(embeds).add_item(self))
def setup(bot: commands.Bot):
    bot.add_cog(DatabaseCOG(bot))

    print(f"Database commands - READY!")


import pandas as pd
class WebullStockData:
    """A class representing stock data obtained from Webull.

    Attributes:
        web_name (str): The name of the stock.
        web_symb (str): The stock's symbol.
        web_exchange (str): The exchange code where the stock is traded.
        web_stock_close (float): The stock's closing price.
        last_earnings (str): The date of the stock's latest earnings report.
        web_stock_vol (int): The stock's trading volume.
        web_change_ratio (float): The stock's price change ratio.
        web_stock_open (float): The stock's opening price.
        web_stock_high (float): The stock's highest price.
        web_stock_low (float): The stock's lowest price.
        fifty_high (float): The stock's 52-week high price.
        avg_vol3m (float): The stock's average trading volume over the past 3 months.
        fifty_low (float): The stock's 52-week low price.
        avg_10d_vol (float): The stock's average trading volume over the past 10 days.
        outstanding_shares (int): The number of outstanding shares of the stock.
        total_shares (int): The total number of shares of the stock.
        estimated_earnings (str): The estimated date of the stock's next earnings report.
        web_vibrate_ratio (float): The stock's price fluctuation ratio.
    """
    def __init__(self, r):
        self.r = r
        for i in self.r:
            self.web_name = i.get("name", None)
            self.web_symb = i.get("symbol", None)
            self.web_exchange = i.get("disExchangeCode", None)
            self.web_stock_close =i.get("close", None)
            self.last_earnings = i.get('latestEarningsDate',None)
            self.web_stock_vol =i.get("volume",None)
            self.web_change_ratio = i.get("changeRatio", None)
            self.web_stock_open =i.get("open",None)
            self.web_stock_high =i.get("high", None)
            self.web_stock_low =i.get("low", None)
            self.fifty_high = i.get("fiftyTwoWkHigh", None)
            self.avg_vol3m = i.get('avgVol3M')
            self.fifty_low = i.get("fiftyTwoWkLow", None)
            self.avg_10d_vol = i.get("avgVol10D", None)
            self.outstanding_shares = i.get('outstandingShares', None)
            self.total_shares = i.get('totalShares', None)

            try:
                self.estimated_earnings = i.get("nextEarningDay", None)
                self.web_vibrate_ratio = i.get('vibrateRatio', None)
            except KeyError:
                self.estimated_earnings = None
                self.web_vibrate_ratio = None


            #self.data_dict = {
            #     'Company Name': i.get("name", None),
            #     'Symbol': i.get("symbol", None),
            #     'Exchange': i.get("disExchangeCode", None),
            #     'Close Price': i.get("close", None),
            #     'Latest Earnings': i.get('latestEarningsDate', None),
            #     'Volume': i.get("volume", None),
            #     'Change Ratio': i.get("changeRatio", None),
            #     'Open Price': i.get("open", None),
            #     'High Price': i.get("high", None),
            #     'Low Price': i.get("low", None),
            #     '52week High': i.get("fiftyTwoWkHigh", None),
            #     'Avg 3month Volume': i.get('avgVol3M', None),
            #     '52week Low': i.get("fiftyTwoWkLow", None),
            #     'Avg 10day Volume': self.r.get("avgVol10D", None),
            #     'Outstanding Shares': self.r.get('outstandingShares', None),
            #     'Total Shares': self.r.get('totalShares', None)
            # }

            # self.df = pd.DataFrame(self.data_dict, index=[0]).transpose()



class MultiQuote:
    def __init__(self, datas):
        self.tickerId = [i.get('tickerId') for i in datas]
        self.exchangeId = [i.get('exchangeId') for i in datas]
        self.type = [i.get('type') for i in datas]
        self.secType = [i.get('secType') for i in datas]
        self.regionId = [i.get('regionId') for i in datas]
        self.regionCode = [i.get('regionCode') for i in datas]
        self.currencyId = [i.get('currencyId') for i in datas]
        self.name = [i.get('name') for i in datas]
        self.symbol = [i.get('symbol') for i in datas]
        self.disSymbol = [i.get('disSymbol') for i in datas]
        self.disExchangeCode = [i.get('disExchangeCode') for i in datas]
        self.exchangeCode = [i.get('exchangeCode') for i in datas]
        self.listStatus = [i.get('listStatus') for i in datas]
        self.template = [i.get('template') for i in datas]
        self.derivativeSupport = [i.get('derivativeSupport') for i in datas]
        self.isPTP = [i.get('isPTP') for i in datas]
        self.filingsSupport = [i.get('filingsSupport') for i in datas]
        self.futuresSupport = [i.get('futuresSupport') for i in datas]
        self.mkTradeTime = [i.get('mkTradeTime') for i in datas]
        self.tradeTime = [i.get('tradeTime') for i in datas]
        self.status = [i.get('status') for i in datas]
        self.close = [i.get('close') for i in datas]
        self.change = [i.get('change') for i in datas]
        self.changeRatio = [i.get('changeRatio') for i in datas]
        self.marketValue = [i.get('marketValue') for i in datas]
        self.volume = [i.get('volume') for i in datas]
        self.turnoverRate = [i.get('turnoverRate') for i in datas]
        self.overnight = [i.get('overnight') for i in datas]
        self.timeZone = [i.get('timeZone') for i in datas]
        self.tzName = [i.get('tzName') for i in datas]
        self.preClose = [i.get('preClose') for i in datas]
        self.open = [i.get('open') for i in datas]
        self.high = [i.get('high') for i in datas]
        self.low = [i.get('low') for i in datas]
        self.vibrateRatio = [i.get('vibrateRatio') for i in datas]
        self.avgVol10D = [i.get('avgVol10D') for i in datas]
        self.avgVol3M = [i.get('avgVol3M') for i in datas]
        self.negMarketValue = [i.get('negMarketValue') for i in datas]
        self.pe = [i.get('pe') for i in datas]
        self.forwardPe = [i.get('forwardPe') for i in datas]
        self.indicatedPe = [i.get('indicatedPe') for i in datas]
        self.peTtm = [i.get('peTtm') for i in datas]
        self.eps = [i.get('eps') for i in datas]
        self.epsTtm = [i.get('epsTtm') for i in datas]
        self.pb = [i.get('pb') for i in datas]
        self.totalShares = [i.get('totalShares') for i in datas]
        self.outstandingShares = [i.get('outstandingShares') for i in datas]
        self.fiftyTwoWkHigh = [i.get('fiftyTwoWkHigh') for i in datas]
        self.fiftyTwoWkLow = [i.get('fiftyTwoWkLow') for i in datas]
        self.dividend = [i.get('dividend') for i in datas]
        self.yield_ = [i.get('yield') for i in datas]
        self.currencyCode = [i.get('currencyCode') for i in datas]
        self.lotSize = [i.get('lotSize') for i in datas]
        self.latestDividendDate = [i.get('latestDividendDate') for i in datas]
        self.latestSplitDate = [i.get('latestSplitDate') for i in datas]
        self.latestEarningsDate = [i.get('latestEarningsDate') for i in datas]
        self.ps = [i.get('ps') for i in datas]
        self.bps = [i.get('bps') for i in datas]
        self.estimateEarningsDate = [i.get('estimateEarningsDate') for i in datas]
        self.tradeStatus = [i.get('tradeStatus') for i in datas]

        self.data_dict = {
            'ticker_id': self.tickerId,
            'exchange_id': self.exchangeId,
            'type': self.type,
            'sec_type': self.secType,
            'region_id': self.regionId,
            'region_code': self.regionCode,
            'currency_id': self.currencyId,
            'name': self.name,
            'symbol': self.symbol,
            'dis_symbol': self.disSymbol,
            'dis_exchange_code': self.disExchangeCode,
            'exchange_code': self.exchangeCode,
            'list_status': self.listStatus,
            'template': self.template,
            'derivative_support': self.derivativeSupport,
            'is_ptp': self.isPTP,
            'filings_support': self.filingsSupport,
            'futures_support': self.futuresSupport,
            'mk_trade_time': self.mkTradeTime,
            'trade_time': self.tradeTime,
            'status': self.status,
            'close': self.close,
            'change': self.change,
            'change_ratio': self.changeRatio,
            'market_value': self.marketValue,
            'volume': self.volume,
            'turnover_rate': self.turnoverRate,
            'overnight': self.overnight,
            'time_zone': self.timeZone,
            'tz_name': self.tzName,
            'pre_close': self.preClose,
            'open': self.open,
            'high': self.high,
            'low': self.low,
            'vibrate_ratio': self.vibrateRatio,
            'avg_vol_10d': self.avgVol10D,
            'avg_vol_3m': self.avgVol3M,
            'neg_market_value': self.negMarketValue,
            'pe': self.pe,
            'forward_pe': self.forwardPe,
            'indicated_pe': self.indicatedPe,
            'pe_ttm': self.peTtm,
            'eps': self.eps,
            'eps_ttm': self.epsTtm,
            'pb': self.pb,
            'total_shares': self.totalShares,
            'outstanding_shares': self.outstandingShares,
            'fifty_two_wk_high': self.fiftyTwoWkHigh,
            'fifty_two_wk_low': self.fiftyTwoWkLow,
            'dividend': self.dividend,
            'yield': self.yield_,
            'currency_code': self.currencyCode,
            'lot_size': self.lotSize,
            'latest_dividend_date': self.latestDividendDate,
            'latest_split_date': self.latestSplitDate,
            'latest_earnings_date': self.latestEarningsDate,
            'ps': self.ps,
            'bps': self.bps,
            'estimate_earnings_date': self.estimateEarningsDate,
            'trade_status': self.tradeStatus
        }


        self.as_dataframe = pd.DataFrame(self.data_dict)




        


import os
from dotenv import load_dotenv
load_dotenv()
import pandas as pd
import requests
from typing import List

from .fedprint_models import Series, Search, Item, SingleItem

class FedPrintSync:
    """
    Queries Federal Reserve documents 

    
    """
    def __init__(self):
        self.headers = {'X-API-Key': os.environ.get('YOUR_FED_PRINT_KEY')}

        self.session = requests.session()

    def series(self):
        url = "https://fedinprint.org/api/series"
        
     
        data = self.session.get(url).json()
        return Series(data)
                

    def search(self, filter:str, limit:str='10', page:str='1'):
        """
        Search Fed Reserve published papers

        ARGUMENTS:

        >>> page: the page to start on (optional - default 1)
        >>> limit: the number of results returned (optional - default 10)
        >>> filter: filter by: (optional - default year)

            - year 
            - provider
            - series
            - contenttype
            - jel
        """
    

  
        url = f"https://fedinprint.org/api/item/search?page={page}&limit={limit}&filter={filter}"
        print(url)

        data = self.session.get(url, headers=self.headers).json()
        print(data)
        search_object = Search(data)  # create a Search object
        return search_object

    
    def get_series_id(self):
        """
        Returms a list of series IDs to be used with the 
        get_item function.
        
        """
        get_series = self.series()
        return get_series.id



    def get_series(self, id):
        """
        Uses the IDS in the series ID list returned from get_series_id

        >>> Argument: id - the id to query
        
        """
       
        url = f"https://fedinprint.org/api/series/{id}/items"
        print(url)
        

        try:
            data = self.session.get(url, headers=self.headers).json()
            series_data = Series(data)  # assuming Series is a class you've defined
            return series_data
        except Exception as e:
            print(f"An error occurred: {e}")
  

    def get_item(self, page:str='1', limit:str='25'):
        """
        Returns a list of all items and the associated item record metadata.

        Arguments:

        >>> page: optional
        >>> limit: optional

        
        """


        url = f"https://fedinprint.org/api/item?page={page}&limit={limit}"
        

        data = self.session.get(url, headers=self.headers).json()
             
        records = data.get('records')

        data = Item(records)
        return data

    def print_file_urls(self, limit:str='25'):
        """
        Returns the URLs to the documents

        Arguments:

        >>> limit: the amount of URLs to return (optional - default 25)
        
        """
        file_urls = self.get_item(limit)
      
        return file_urls.fileurl



import disnake
from disnake.ext import commands

# cogs/main_view.py
from .earnings import EarningsSelect
from .gex import GexSelect
from .options import OptsSelect
import asyncio

class MainView(disnake.ui.View):
    def __init__(self, bot = None, ticker=None):
        self.ticker=ticker
        self.bot=bot
        super().__init__(timeout=None)
        
        # Initialize other views as needed


    @disnake.ui.button(style=disnake.ButtonStyle.blurple, label='GEX', row=0, custom_id='gexcmds')
    async def gex(self, button:disnake.ui.Button, inter:disnake.AppCmdInter):
        self.clear_items()
        self.add_item(GexSelect())

        

    @disnake.ui.button(style=disnake.ButtonStyle.blurple, label='EARNINGS', row=0, custom_id='earnings')
    async def earnings(self, button:disnake.ui.Button, inter:disnake.AppCmdInter):
        self.clear_items()
        self.add_item(EarningsSelect())




        # Clear the existing items and add a new OptsSelect with the ticker
        self.clear_items()
        self.add_item(OptsSelect(self.ticker))




    @disnake.ui.button(style=disnake.ButtonStyle.blurple, label='OPTIONS',row=0, custom_id='options')
    async def opts(self, button:disnake.ui.Button, inter:disnake.AppCmdInter):
        self.clear_items()
        await inter.response.defer()
        # Check if ticker is None and prompt the user
        if self.ticker is None:
            await inter.send("Please enter the ticker:")

            # Wait for a message from the user; you can add checks to validate the response
            try:
                message = await self.bot.wait_for(
                    'message',
                    check=lambda m: m.author == inter.author and m.channel == inter.channel,
                    timeout=30.0  # Timeout after 30 seconds
                )
            except asyncio.TimeoutError:
                await inter.followup.send("You didn't reply in time!")
                return

            # Set the ticker with the content of the message
            self.ticker = message.content


            self.add_item(OptsSelect(ticker=self.ticker))

            await inter.send(view=self)


def setup(bot: commands.Bot):
    bot.add_cog(MainView(bot))
    print('MainView')


import sys
from pathlib import Path
# Add the project directory to the sys.path
project_dir = str(Path(__file__).resolve().parents[3])
if project_dir not in sys.path:
    sys.path.append(project_dir)
import os
import aiohttp
import csv
import asyncio
import requests
from discord_webhook import AsyncDiscordWebhook, DiscordEmbed
from concurrent.futures import ThreadPoolExecutor
from datetime import datetime, timedelta
from dotenv import load_dotenv
load_dotenv()
from conditional_scans import ConditionalScans
from apis.webull.webull_markets import WebullMarkets
from apis.webull.webull_trading import WebullTrading
from apis.polygonio.async_polygon_sdk import Polygon
from apis.polygonio.polygon_options import PolygonOptions
from _markets.list_sets.ticker_lists import most_active_tickers
from discord_.embeddings import Embeddings
from discord_webhook import AsyncDiscordWebhook, DiscordEmbed




class ReferenceMarket(Embeddings):
    def __init__(self):
        
        self.today = datetime.now().strftime('%Y-%m-%d')
        self.yesterday = (datetime.now() - timedelta(days=1)).strftime('%Y-%m-%d')
        self.tomorrow = (datetime.now() + timedelta(days=1)).strftime('%Y-%m-%d')
        self.thirty_days_ago = (datetime.now() - timedelta(days=30)).strftime('%Y-%m-%d')
        self.thirty_days_from_now = (datetime.now() + timedelta(days=30)).strftime('%Y-%m-%d')
        self.fifteen_days_ago = (datetime.now() - timedelta(days=15)).strftime('%Y-%m-%d')
        self.fifteen_days_from_now = (datetime.now() + timedelta(days=15)).strftime('%Y-%m-%d')
        self.eight_days_from_now = (datetime.now() + timedelta(days=8)).strftime('%Y-%m-%d')
        self.eight_days_ago = (datetime.now() - timedelta(days=8)).strftime('%Y-%m-%d')     
   
        #conditions

        self.accumulation = os.environ.get('accumulation')
        self.fire_sale = os.environ.get('fire_sale')
        self.neutral_zone = os.environ.get('neutral_zone')
        self.trading = WebullTrading()
        self.markets = WebullMarkets()
        self.poly = Polygon(connection_string=os.environ.get('POLYGON_STRING'))
        self.opts = PolygonOptions(user='chuck', database='markets')
        self.conditions = ConditionalScans()







    async def volume_analysis_screener(self, ticker:str):

        """
        Scan for fire sale conditions - where the overall  volume on the day is 70% or more sell volume.


        CONDITIONS:


        >>> fire_sale: volume on the day is recorded as 70% or more sell volume.

        >>> accumulation: volume on the day is recorded as 70% or more buy volume.


        >>> neutral_zone: volume on the day is recorded as 70% or more neutral volume.
        
        
        """

        volume_analysis = await self.trading.volume_analysis(ticker)
        data_dict = getattr(volume_analysis, 'data_dict', None)
        print(ticker)
        if volume_analysis is not None:
            if data_dict is not None:
                data_dict.update({'ticker': ticker})
            if volume_analysis.buyPct >= 70:
                condition = 'accumulation'

                await self.volume_analysis_embed(condition, self.accumulation, data_dict)
                

            
            if volume_analysis.sellPct >= 70:
                condition = 'fire_sale'
                await self.volume_analysis_embed(condition, self.fire_sale, data_dict)


            if volume_analysis.nPct >= 70:
                condition = 'neutral_zone'
                await self.volume_analysis_embed(condition,self.neutral_zone, data_dict)


    async def run_markets(self):
        all_data = await self.poly.get_all_tickers()

        min_opens = all_data.min_o
        min_closes = all_data.min_c
        tickers = all_data.ticker

        for ticker, open_value, close_value in zip(tickers, min_opens, min_closes):
            # Calculate the percentage difference
            if open_value and close_value:  # Ensure neither value is None or zero to avoid division errors
                percentage_difference = abs(close_value - open_value) / open_value * 100

                # Check if the percentage difference is 1% or more
                if percentage_difference >= 1:
                    print(f"{ticker}'s MINUTE Open ({open_value}) and Close ({close_value}) are more than 1% apart!")

                    hook = AsyncDiscordWebhook("https://discord.com/api/webhooks/1174715581279252591/1DM9fuN6-uG_CwVnXZQtN2tB9tHrnTCP4jhAMAnuYoS38mTnGvwDjVziTzYX8F8cjT3y", content=f"{ticker}'s MINUTE Open ({open_value}) and Close ({close_value}) are more than 1% apart!")

                    await hook.execute()

    async def refresh_options(self, ticker):
        data = await self.opts.get_option_chain_all(underlying_asset=ticker)
        headers = [
            'strike', 'expiry', 'dte', 'time_value', 'moneyness', 'liquidity_score', 'cp', 
            'exercise_style', 'option_symbol', 'theta', 'theta_decay_rate', 'delta', 
            'delta_theta_ratio', 'gamma', 'gamma_risk', 'vega', 'vega_impact', 'timestamp', 
            'oi', 'open', 'high', 'low', 'close', 'intrinstic_value', 'extrinsic_value', 
            'leverage_ratio', 'vwap', 'conditions', 'price', 'trade_size', 'exchange', 'ask', 
            'bid', 'spread', 'spread_pct', 'iv', 'bid_size', 'ask_size', 'vol', 'mid', 
            'change_to_breakeven', 'underlying_price', 'ticker', 'return_on_risk', 'velocity', 
            'sensitivity', 'greeks_balance', 'opp'
        ]

        if data is not None:
            # Open a CSV file to write data
            with open('all_options_data.csv', mode='a', newline='', encoding='utf-8') as file:
                writer = csv.writer(file)
                writer.writerow(headers)

                # Write each row to the CSV file within the 'with' block
                for i,row in data.df.iterrows():
                    writer.writerow(row)
                    

market = ReferenceMarket()
async def refresh_markets():
    # Initialize tasks with volume_analysis_screener for each ticker
    #tasks = [market.volume_analysis_screener(ticker) for ticker in most_active_tickers]
    tasks = []
    # Add run_markets task
    #tasks.append(market.run_markets())
    while True:
        # Add run_options tasks for each ticker
        tasks.extend([market.refresh_options(ticker) for ticker in most_active_tickers])
        tasks.append(market.run_markets())
        # Use asyncio.gather to run all tasks concurrently
        await asyncio.gather(*tasks)

### ANALYZE

import pandas as pd
from apis.helpers import human_readable
df = pd.read_csv('files/all_option_data.csv')



# Randomize the DataFrame
df = df.sample(frac=1).reset_index(drop=True)

def process_row(row):
    # Your processing logic for each row
    ask = row['ask']
    bid = row['bid']
    mid = row['mid']
    spread = row['spread']
    spread_pct = row['spread_pct']
    ask_size=  row['ask_size']
    bid_size = row['bid_size']


    theta = row['theta']
    theta_decay_rate = row['theta_decay_rate']
    vega = row['vega']
    vega_impact = row['vega_impact']
    gamma = row['gamma']
    gamma_risk = row['gamma_risk']
    delta = row['delta']
    delta_theta_ratio = row['delta_theta_ratio']

    intrinsic_value = row['intrinstic_value']
    extrinsic_value = row['extrinsic_value']
    time_value = row['time_value']

    vol = row['vol']
    oi = row['oi']
    iv = row['iv']
    liquidity_score = row['liquidity_score']


    change_ratio = row['change_ratio']
    velocity = row['velocity']



    open = row['open']
    high = row['high']
    low = row['low']
    close = row['close']
    vwap = row['vwap']






    dte = row['dte']
    price = row['underlying_price']
    # Assuming human_readable is a function you've defined
    symbol = human_readable(row['option_symbol'])
    strike = row['strike']
    cp = row['cp']
    expiry = row['expiry']
    
    
    results = market.conditions.refined_options_analysis(row)
    if results is not None:
        if results['theta_risk'] == 'Low':
            print(row)    
    
async def process_data_asynchronously(executor, df):
    loop = asyncio.get_running_loop()

    for _, row in df.iterrows():
        # Schedule each row to be processed individually
        await loop.run_in_executor(executor, process_row, row)

async def main():
    with ThreadPoolExecutor() as executor:
        await process_data_asynchronously(executor, df)

# Run the async function with multi-threading
asyncio.run(main())


import os
from dotenv import load_dotenv
load_dotenv()
import aiohttp
import asyncio
import pandas as pd
from datetime import datetime
import asyncpg

class AggsModel:
    def __init__(self):
        self.api_key = os.environ.get('YOUR_POLYGON_KEY')
        self.today = datetime.now().strftime('%Y-%m-%d')
    async def fetch_stock_data(self, date=None, adjusted=True, include_otc=False):
        if date is None:
            date = self.today
        url = f"https://api.polygon.io/v2/aggs/grouped/locale/us/market/stocks/{date}"
        params = {
            "adjusted": str(adjusted).lower(),
            "include_otc": str(include_otc).lower(),
            "apiKey": self.api_key
        }
        async with aiohttp.ClientSession() as session:
            async with session.get(url, params=params) as response:
                data = await response.json()
                df = pd.DataFrame(data['results'])
                df['timestamp'] = pd.to_datetime(df['t'], unit='ms')  # Adjust the unit if necessary
                return df.drop(columns=['t'])

    async def create_table(self, conn):
        await conn.execute("""
            CREATE TABLE IF NOT EXISTS daily_aggregates (
                T TEXT,
                V BIGINT,
                VW REAL,
                O REAL,
                C REAL,
                H REAL,
                L REAL,
                N BIGINT,
                Timestamp TIMESTAMP WITHOUT TIME ZONE
            )
        """)

    async def insert_data(self, conn, data_frame):
        for index, row in data_frame.iterrows():
            await conn.execute("""
                INSERT INTO daily_aggregates (T, V, VW, O, C, H, L, N, Timestamp) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9)
            """, row['T'], row['v'], row['vw'], row['o'], row['c'], row['h'], row['l'], row['n'], row['timestamp'])

aggs_model = AggsModel()
loop = asyncio.get_event_loop()
connection = loop.run_until_complete(asyncpg.connect(user=os.environ.get('DB_USER'), password=os.environ.get('DB_PASSWORD'), database='polygon', host=os.environ.get('DB_HOST')))
loop.run_until_complete(aggs_model.create_table(connection))
df = loop.run_until_complete(aggs_model.fetch_stock_data())
loop.run_until_complete(aggs_model.insert_data(connection, df))


from dataclasses import dataclass, asdict
from datetime import datetime

import pandas as pd

@dataclass
class DailyTreasuryData:
    date: datetime
    close_balance: float
    open_balance: float
    amount_change: float
    percent_change: float
    moving_avg: float

    def as_dataframe(self) -> pd.DataFrame:
        """Converts the object to a pandas DataFrame."""
        return pd.DataFrame([asdict(self)])
    

    def insert_into_pg(self, conn):
        cursor = conn.cursor()
        insert_query = """INSERT INTO daily_treasury (date, close_balance, open_balance, amount_change, percent_change, moving_avg) VALUES (%s, %s, %s, %s, %s, %s);"""
        cursor.execute(insert_query, (self.date, self.close_balance, self.open_balance, self.amount_change, self.percent_change, self.moving_avg))
        conn.commit()



from dataclasses import dataclass, asdict
from datetime import datetime

import pandas as pd

@dataclass
class RetailSalesData:
    date: datetime
    amount: float
    percent_change: float
    monthly_avg_cases: int


    def as_dataframe(self) -> pd.DataFrame:
        """Converts the object to a pandas DataFrame."""
        return pd.DataFrame([asdict(self)]) 

    def insert_into_pg(self, conn):
        cursor = conn.cursor()
        insert_query = """INSERT INTO retail_sales (date, amount, percent_change, monthly_avg_cases) VALUES (%s, %s, %s, %s);"""
        cursor.execute(insert_query, (self.date, self.amount, self.percent_change, self.monthly_avg_cases))
        conn.commit()



import os
from dotenv import load_dotenv
import requests
import pandas as pd
from .oic_models import OICOptionsMonitor
from ..polygonio.polygon_options import PolygonOptions
db_config = {
    "host": os.environ.get('DB_HOST'), # Default to this IP if 'DB_HOST' not found in environment variables
    "port": int(os.environ.get('DB_PORT')), # Default to 5432 if 'DB_PORT' not found
    "user": os.environ.get('DB_USER'), # Default to 'postgres' if 'DB_USER' not found
    "password": os.environ.get('DB_PASSWORD'), # Use the password from environment variable or default
    "database": os.environ.get('DB_NAME') # Database name for the new jawless database
}
opts = PolygonOptions()
import time
load_dotenv()


class OICSDK:
    def __init__(self):
        self.session = requests.Session()
        self.refresh_key_payload = {
            "clientKey": f"{os.environ.get('OIC_KEY')}",
            "clientName": "OIC_test"
        }
        self.token = None

        self.initialize_token()


    def initialize_token(self):
        """
        Initialize the token by making the first request.
        """
        r = self.session.post(
            "https://ivlivefront.ivolatility.com/token/client/get",
            json=self.refresh_key_payload
        )
        if r.status_code == 200:
            self.token = r.text # Adjust the key according to actual response structure

    def build_headers(self):
        """
        Build headers using the current token.
        """
        if not self.token:
            self.initialize_token()
        return {'Authorization': f'Bearer {self.token}'}

    def refresh_token(self):
        """
        Refresh the token.
        """
        r = self.session.post(
            "https://ivlivefront.ivolatility.com/token/client/get",
            headers=self.build_headers(),
            json=self.refresh_key_payload
        )
        if r.status_code == 200:
            self.token = r
        return self.token
    def get_option_id(self, ticker):
        """
        Gets the option ID using the lookup endpoint
        
        """
        url = f"https://private-authorization.ivolatility.com/lookup/?sortField=SYMBOL&symbol={ticker}&region=1&matchingType=EXACTLY&pageSize=10&page=0"

        r = self.session.get(url, headers=self.build_headers()).json()
        pages = r['page']
        id = pages[0]['stockId']
        return id
    def most_active_options(self):
        """
        Gets the most active options from the Options Industry Council
        
        """
        url=f"https://private-authorization.ivolatility.com/favorites/instruments/most-active"
        data = self.session.get(url, headers=self.build_headers()).json()


        print(data)

        df = pd.DataFrame(data)
        return df

    def get_price(self, ticker):
        r = requests.get(f"https://api.polygon.io/v2/last/nbbo/{ticker}?apiKey={os.environ.get('YOUR_POLYGON_KEY')}").json()
        results = r['results'] if 'results' in r else None
        if results is not None:
            price = results['P']
            print(price)
            return price

    async def options_monitor(self, ticker):
        await opts.connect()
        stock_id = self.get_option_id(ticker)
        stock_price = self.get_price(ticker)
        print(stock_price)

        url = f"https://private-authorization.ivolatility.com/options-monitor/listOptionDataRow?stockId={stock_id}&center={stock_price}&regionId=1&strikesN=75&columns=alpha&columns=ask&columns=asksize&columns=asktime&columns=bid&columns=bidsize&columns=bidtime&columns=change&columns=changepercent&columns=delta&columns=theoprice&columns=gamma&columns=ivint&columns=ivask&columns=ivbid&columns=mean&columns=openinterest_eod&columns=optionsymbol&columns=volume&columns=paramvolapercent_eod&columns=alpha_eod&columns=ask_eod&columns=bid_eod&columns=delta_eod&columns=theoprice_eod&columns=gamma_eod&columns=ivint_eod&columns=mean_eod&columns=rho_eod&columns=theta_eod&columns=vega_eod&columns=changepercent_eod&columns=change_eod&columns=volume_eod&columns=quotetime&columns=rho&columns=strike&columns=style&columns=theta&columns=vega&columns=expirationdate&columns=forwardprice&columns=forwardprice_eod&columns=days&columns=days_eod&columns=iv&columns=iv_eod&rtMode=RT&userId=9999999&uuid=null"
        print(url)

        # Add your logic here to process the URL
        # For example, making an HTTP request

        r = requests.get(url, headers=self.build_headers()).json()


        data = OICOptionsMonitor(r)
        try:
            df = data.as_dataframe
            await opts.batch_insert_dataframe(df, table_name='monitor', unique_columns='insertion_timestamp', batch_size=1)
        except Exception as e:
            print(e)
    
      
        return data
    

    def iv_monitor(self):
        url = f"https://.ivolatility.com/ivx-monitor"
        r = requests.get(url, headers=self.build_headers()).json()
        return r







from fudstop.apis.federal_register.fed_register_sdk import FedRegisterSDK



register = FedRegisterSDK()



import asyncio



async def main():

    x = await register.query_document('options')

    print(x.data_dict)
asyncio.run(main())